# Form implementation generated from reading ui file 'DragonLog_QSOForm.ui'
#
# Created by: PyQt6 UI code generator 6.9.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_QSOForm(object):
    def setupUi(self, QSOForm):
        QSOForm.setObjectName("QSOForm")
        QSOForm.resize(450, 504)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(QSOForm.sizePolicy().hasHeightForWidth())
        QSOForm.setSizePolicy(sizePolicy)
        QSOForm.setMinimumSize(QtCore.QSize(450, 480))
        self.verticalLayout = QtWidgets.QVBoxLayout(QSOForm)
        self.verticalLayout.setObjectName("verticalLayout")
        self.toolBox = QtWidgets.QToolBox(parent=QSOForm)
        self.toolBox.setObjectName("toolBox")
        self.mainPage = QtWidgets.QWidget()
        self.mainPage.setGeometry(QtCore.QRect(0, 0, 432, 334))
        self.mainPage.setObjectName("mainPage")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.mainPage)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.nameLineEdit = QtWidgets.QLineEdit(parent=self.mainPage)
        self.nameLineEdit.setProperty("cssClass", "QSOFormInput")
        self.nameLineEdit.setObjectName("nameLineEdit")
        self.gridLayout_3.addWidget(self.nameLineEdit, 0, 1, 1, 1)
        self.locatorLineEdit = QtWidgets.QLineEdit(parent=self.mainPage)
        self.locatorLineEdit.setProperty("cssClass", "QSOFormInput")
        self.locatorLineEdit.setObjectName("locatorLineEdit")
        self.gridLayout_3.addWidget(self.locatorLineEdit, 1, 1, 1, 1)
        self.callSignLineEdit = QtWidgets.QLineEdit(parent=self.mainPage)
        self.callSignLineEdit.setProperty("cssClass", "QSOFormInput")
        self.callSignLineEdit.setObjectName("callSignLineEdit")
        self.gridLayout_3.addWidget(self.callSignLineEdit, 0, 0, 1, 1)
        self.QTHLineEdit = QtWidgets.QLineEdit(parent=self.mainPage)
        self.QTHLineEdit.setProperty("cssClass", "QSOFormInput")
        self.QTHLineEdit.setObjectName("QTHLineEdit")
        self.gridLayout_3.addWidget(self.QTHLineEdit, 1, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_3.addItem(spacerItem, 2, 0, 1, 1)
        self.verticalLayout_8.addLayout(self.gridLayout_3)
        self.horizontalLayout_11.addLayout(self.verticalLayout_8)
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.verticalLayout_7.setSpacing(0)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.searchHamQTHPushButton = QtWidgets.QPushButton(parent=self.mainPage)
        self.searchHamQTHPushButton.setObjectName("searchHamQTHPushButton")
        self.verticalLayout_7.addWidget(self.searchHamQTHPushButton)
        self.searchQRZPushButton = QtWidgets.QPushButton(parent=self.mainPage)
        self.searchQRZPushButton.setObjectName("searchQRZPushButton")
        self.verticalLayout_7.addWidget(self.searchQRZPushButton)
        self.searchQRZCQPushButton = QtWidgets.QPushButton(parent=self.mainPage)
        self.searchQRZCQPushButton.setObjectName("searchQRZCQPushButton")
        self.verticalLayout_7.addWidget(self.searchQRZCQPushButton)
        self.horizontalLayout_11.addLayout(self.verticalLayout_7)
        self.verticalLayout_2.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.ctyCtyLabel = QtWidgets.QLabel(parent=self.mainPage)
        self.ctyCtyLabel.setText("")
        self.ctyCtyLabel.setObjectName("ctyCtyLabel")
        self.horizontalLayout_10.addWidget(self.ctyCtyLabel)
        self.line_4 = QtWidgets.QFrame(parent=self.mainPage)
        self.line_4.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_4.setObjectName("line_4")
        self.horizontalLayout_10.addWidget(self.line_4)
        self.ctyAreaLabel = QtWidgets.QLabel(parent=self.mainPage)
        self.ctyAreaLabel.setText("")
        self.ctyAreaLabel.setObjectName("ctyAreaLabel")
        self.horizontalLayout_10.addWidget(self.ctyAreaLabel)
        self.line_5 = QtWidgets.QFrame(parent=self.mainPage)
        self.line_5.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_5.setObjectName("line_5")
        self.horizontalLayout_10.addWidget(self.line_5)
        self.distLabel = QtWidgets.QLabel(parent=self.mainPage)
        self.distLabel.setText("")
        self.distLabel.setObjectName("distLabel")
        self.horizontalLayout_10.addWidget(self.distLabel)
        self.verticalLayout_2.addLayout(self.horizontalLayout_10)
        self.line = QtWidgets.QFrame(parent=self.mainPage)
        self.line.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_2.addWidget(self.line)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label = QtWidgets.QLabel(parent=self.mainPage)
        self.label.setObjectName("label")
        self.horizontalLayout_9.addWidget(self.label)
        self.RSTSentLineEdit = QtWidgets.QLineEdit(parent=self.mainPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.RSTSentLineEdit.sizePolicy().hasHeightForWidth())
        self.RSTSentLineEdit.setSizePolicy(sizePolicy)
        self.RSTSentLineEdit.setMaximumSize(QtCore.QSize(100, 16777215))
        self.RSTSentLineEdit.setProperty("cssClass", "QSOFormInput")
        self.RSTSentLineEdit.setObjectName("RSTSentLineEdit")
        self.horizontalLayout_9.addWidget(self.RSTSentLineEdit)
        self.label_2 = QtWidgets.QLabel(parent=self.mainPage)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_9.addWidget(self.label_2)
        self.RSTRcvdLineEdit = QtWidgets.QLineEdit(parent=self.mainPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.RSTRcvdLineEdit.sizePolicy().hasHeightForWidth())
        self.RSTRcvdLineEdit.setSizePolicy(sizePolicy)
        self.RSTRcvdLineEdit.setMaximumSize(QtCore.QSize(100, 16777215))
        self.RSTRcvdLineEdit.setProperty("cssClass", "QSOFormInput")
        self.RSTRcvdLineEdit.setObjectName("RSTRcvdLineEdit")
        self.horizontalLayout_9.addWidget(self.RSTRcvdLineEdit)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem1)
        self.powerLabel = QtWidgets.QLabel(parent=self.mainPage)
        self.powerLabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.powerLabel.setObjectName("powerLabel")
        self.horizontalLayout_9.addWidget(self.powerLabel)
        self.powerSpinBox = QtWidgets.QSpinBox(parent=self.mainPage)
        self.powerSpinBox.setEnabled(True)
        self.powerSpinBox.setMinimum(0)
        self.powerSpinBox.setMaximum(1000)
        self.powerSpinBox.setProperty("cssClass", "QSOFormInput")
        self.powerSpinBox.setObjectName("powerSpinBox")
        self.horizontalLayout_9.addWidget(self.powerSpinBox)
        self.verticalLayout_2.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.bandComboBox = QtWidgets.QComboBox(parent=self.mainPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bandComboBox.sizePolicy().hasHeightForWidth())
        self.bandComboBox.setSizePolicy(sizePolicy)
        self.bandComboBox.setMaximumSize(QtCore.QSize(100, 16777215))
        self.bandComboBox.setEditable(True)
        self.bandComboBox.setMinimumContentsLength(4)
        self.bandComboBox.setProperty("cssClass", "QSOFormInput")
        self.bandComboBox.setObjectName("bandComboBox")
        self.horizontalLayout_8.addWidget(self.bandComboBox)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem2)
        self.channelLabel = QtWidgets.QLabel(parent=self.mainPage)
        self.channelLabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.channelLabel.setObjectName("channelLabel")
        self.horizontalLayout_8.addWidget(self.channelLabel)
        self.channelComboBox = QtWidgets.QComboBox(parent=self.mainPage)
        self.channelComboBox.setProperty("cssClass", "QSOFormInput")
        self.channelComboBox.setObjectName("channelComboBox")
        self.horizontalLayout_8.addWidget(self.channelComboBox)
        self.freqLabel = QtWidgets.QLabel(parent=self.mainPage)
        self.freqLabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.freqLabel.setObjectName("freqLabel")
        self.horizontalLayout_8.addWidget(self.freqLabel)
        self.freqDoubleSpinBox = QtWidgets.QDoubleSpinBox(parent=self.mainPage)
        self.freqDoubleSpinBox.setAccelerated(True)
        self.freqDoubleSpinBox.setDecimals(3)
        self.freqDoubleSpinBox.setMinimum(100.0)
        self.freqDoubleSpinBox.setMaximum(7500000000.0)
        self.freqDoubleSpinBox.setProperty("cssClass", "QSOFormInput")
        self.freqDoubleSpinBox.setObjectName("freqDoubleSpinBox")
        self.horizontalLayout_8.addWidget(self.freqDoubleSpinBox)
        self.verticalLayout_2.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.modeComboBox = QtWidgets.QComboBox(parent=self.mainPage)
        self.modeComboBox.setEditable(True)
        self.modeComboBox.setMinimumContentsLength(2)
        self.modeComboBox.setProperty("cssClass", "QSOFormInput")
        self.modeComboBox.setObjectName("modeComboBox")
        self.horizontalLayout_14.addWidget(self.modeComboBox)
        self.submodeComboBox = QtWidgets.QComboBox(parent=self.mainPage)
        self.submodeComboBox.setProperty("cssClass", "QSOFormInput")
        self.submodeComboBox.setObjectName("submodeComboBox")
        self.horizontalLayout_14.addWidget(self.submodeComboBox)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_14.addItem(spacerItem3)
        self.propComboBox = QtWidgets.QComboBox(parent=self.mainPage)
        self.propComboBox.setProperty("cssClass", "QSOFormInput")
        self.propComboBox.setObjectName("propComboBox")
        self.horizontalLayout_14.addWidget(self.propComboBox)
        self.verticalLayout_2.addLayout(self.horizontalLayout_14)
        self.line_2 = QtWidgets.QFrame(parent=self.mainPage)
        self.line_2.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_2.addWidget(self.line_2)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.timeNowPushButton = QtWidgets.QPushButton(parent=self.mainPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Maximum, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.timeNowPushButton.sizePolicy().hasHeightForWidth())
        self.timeNowPushButton.setSizePolicy(sizePolicy)
        self.timeNowPushButton.setObjectName("timeNowPushButton")
        self.gridLayout.addWidget(self.timeNowPushButton, 0, 3, 1, 1)
        spacerItem4 = QtWidgets.QSpacerItem(100, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem4, 0, 4, 1, 1)
        self.label_3 = QtWidgets.QLabel(parent=self.mainPage)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 0, 0, 1, 1)
        self.timeOffEdit = QtWidgets.QLineEdit(parent=self.mainPage)
        self.timeOffEdit.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.timeOffEdit.sizePolicy().hasHeightForWidth())
        self.timeOffEdit.setSizePolicy(sizePolicy)
        self.timeOffEdit.setProperty("cssClass", "QSOFormInput")
        self.timeOffEdit.setObjectName("timeOffEdit")
        self.gridLayout.addWidget(self.timeOffEdit, 1, 2, 1, 1)
        self.dateOnEdit = QtWidgets.QDateEdit(parent=self.mainPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dateOnEdit.sizePolicy().hasHeightForWidth())
        self.dateOnEdit.setSizePolicy(sizePolicy)
        self.dateOnEdit.setMinimumSize(QtCore.QSize(100, 0))
        self.dateOnEdit.setCalendarPopup(True)
        self.dateOnEdit.setProperty("cssClass", "QSOFormInput")
        self.dateOnEdit.setObjectName("dateOnEdit")
        self.gridLayout.addWidget(self.dateOnEdit, 0, 1, 1, 1)
        self.label_4 = QtWidgets.QLabel(parent=self.mainPage)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 1, 0, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(100, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem5, 1, 4, 1, 1)
        self.dateOffEdit = QtWidgets.QDateEdit(parent=self.mainPage)
        self.dateOffEdit.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dateOffEdit.sizePolicy().hasHeightForWidth())
        self.dateOffEdit.setSizePolicy(sizePolicy)
        self.dateOffEdit.setMinimumSize(QtCore.QSize(100, 0))
        self.dateOffEdit.setCalendarPopup(True)
        self.dateOffEdit.setTimeSpec(QtCore.Qt.TimeSpec.UTC)
        self.dateOffEdit.setProperty("cssClass", "QSOFormInput")
        self.dateOffEdit.setObjectName("dateOffEdit")
        self.gridLayout.addWidget(self.dateOffEdit, 1, 1, 1, 1)
        self.autoDateCheckBox = QtWidgets.QCheckBox(parent=self.mainPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.autoDateCheckBox.sizePolicy().hasHeightForWidth())
        self.autoDateCheckBox.setSizePolicy(sizePolicy)
        self.autoDateCheckBox.setChecked(True)
        self.autoDateCheckBox.setObjectName("autoDateCheckBox")
        self.gridLayout.addWidget(self.autoDateCheckBox, 1, 3, 1, 1)
        self.timeOnEdit = QtWidgets.QLineEdit(parent=self.mainPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.timeOnEdit.sizePolicy().hasHeightForWidth())
        self.timeOnEdit.setSizePolicy(sizePolicy)
        self.timeOnEdit.setProperty("cssClass", "QSOFormInput")
        self.timeOnEdit.setObjectName("timeOnEdit")
        self.gridLayout.addWidget(self.timeOnEdit, 0, 2, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.line_3 = QtWidgets.QFrame(parent=self.mainPage)
        self.line_3.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout_2.addWidget(self.line_3)
        self.commentLineEdit = QtWidgets.QLineEdit(parent=self.mainPage)
        self.commentLineEdit.setProperty("cssClass", "QSOFormInput")
        self.commentLineEdit.setObjectName("commentLineEdit")
        self.verticalLayout_2.addWidget(self.commentLineEdit)
        self.verticalLayout_5.addLayout(self.verticalLayout_2)
        spacerItem6 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        self.verticalLayout_5.addItem(spacerItem6)
        self.toolBox.addItem(self.mainPage, "")
        self.stationPage = QtWidgets.QWidget()
        self.stationPage.setGeometry(QtCore.QRect(0, 0, 432, 334))
        self.stationPage.setObjectName("stationPage")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.stationPage)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.identityGroupBox = QtWidgets.QGroupBox(parent=self.stationPage)
        self.identityGroupBox.setCheckable(True)
        self.identityGroupBox.setObjectName("identityGroupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.identityGroupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.ownCallSignLineEdit = QtWidgets.QLineEdit(parent=self.identityGroupBox)
        self.ownCallSignLineEdit.setEnabled(False)
        self.ownCallSignLineEdit.setProperty("cssClass", "QSOFormInput")
        self.ownCallSignLineEdit.setObjectName("ownCallSignLineEdit")
        self.horizontalLayout_5.addWidget(self.ownCallSignLineEdit)
        self.ownNameLineEdit = QtWidgets.QLineEdit(parent=self.identityGroupBox)
        self.ownNameLineEdit.setEnabled(False)
        self.ownNameLineEdit.setProperty("cssClass", "QSOFormInput")
        self.ownNameLineEdit.setObjectName("ownNameLineEdit")
        self.horizontalLayout_5.addWidget(self.ownNameLineEdit)
        self.verticalLayout_3.addLayout(self.horizontalLayout_5)
        self.verticalLayout_10.addWidget(self.identityGroupBox)
        self.stationGroupBox = QtWidgets.QGroupBox(parent=self.stationPage)
        self.stationGroupBox.setCheckable(True)
        self.stationGroupBox.setObjectName("stationGroupBox")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.stationGroupBox)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.ownQTHComboBox = QtWidgets.QComboBox(parent=self.stationGroupBox)
        self.ownQTHComboBox.setEnabled(False)
        self.ownQTHComboBox.setAutoFillBackground(True)
        self.ownQTHComboBox.setEditable(True)
        self.ownQTHComboBox.setProperty("cssClass", "QSOFormInput")
        self.ownQTHComboBox.setObjectName("ownQTHComboBox")
        self.horizontalLayout_7.addWidget(self.ownQTHComboBox)
        self.verticalLayout_4.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.radioComboBox = QtWidgets.QComboBox(parent=self.stationGroupBox)
        self.radioComboBox.setEnabled(False)
        self.radioComboBox.setEditable(True)
        self.radioComboBox.setProperty("cssClass", "QSOFormInput")
        self.radioComboBox.setObjectName("radioComboBox")
        self.horizontalLayout_6.addWidget(self.radioComboBox)
        self.antennaComboBox = QtWidgets.QComboBox(parent=self.stationGroupBox)
        self.antennaComboBox.setEnabled(False)
        self.antennaComboBox.setEditable(True)
        self.antennaComboBox.setProperty("cssClass", "QSOFormInput")
        self.antennaComboBox.setObjectName("antennaComboBox")
        self.horizontalLayout_6.addWidget(self.antennaComboBox)
        self.verticalLayout_4.addLayout(self.horizontalLayout_6)
        self.verticalLayout_10.addWidget(self.stationGroupBox)
        self.remarksTextEdit = QtWidgets.QTextEdit(parent=self.stationPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.remarksTextEdit.sizePolicy().hasHeightForWidth())
        self.remarksTextEdit.setSizePolicy(sizePolicy)
        self.remarksTextEdit.setMinimumSize(QtCore.QSize(0, 50))
        self.remarksTextEdit.setMaximumSize(QtCore.QSize(16777215, 100))
        self.remarksTextEdit.setAcceptRichText(False)
        self.remarksTextEdit.setTextInteractionFlags(QtCore.Qt.TextInteractionFlag.LinksAccessibleByKeyboard|QtCore.Qt.TextInteractionFlag.LinksAccessibleByMouse|QtCore.Qt.TextInteractionFlag.TextBrowserInteraction|QtCore.Qt.TextInteractionFlag.TextEditable|QtCore.Qt.TextInteractionFlag.TextEditorInteraction|QtCore.Qt.TextInteractionFlag.TextSelectableByKeyboard|QtCore.Qt.TextInteractionFlag.TextSelectableByMouse)
        self.remarksTextEdit.setProperty("cssClass", "QSOFormInput")
        self.remarksTextEdit.setObjectName("remarksTextEdit")
        self.verticalLayout_10.addWidget(self.remarksTextEdit)
        spacerItem7 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        self.verticalLayout_10.addItem(spacerItem7)
        self.toolBox.addItem(self.stationPage, "")
        self.qslPage = QtWidgets.QWidget()
        self.qslPage.setGeometry(QtCore.QRect(0, 0, 432, 334))
        self.qslPage.setObjectName("qslPage")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.qslPage)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.qslViaLineEdit = QtWidgets.QLineEdit(parent=self.qslPage)
        self.qslViaLineEdit.setProperty("cssClass", "QSOFormInput")
        self.qslViaLineEdit.setObjectName("qslViaLineEdit")
        self.verticalLayout_6.addWidget(self.qslViaLineEdit)
        self.qslMessageTextEdit = QtWidgets.QTextEdit(parent=self.qslPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.qslMessageTextEdit.sizePolicy().hasHeightForWidth())
        self.qslMessageTextEdit.setSizePolicy(sizePolicy)
        self.qslMessageTextEdit.setMinimumSize(QtCore.QSize(0, 50))
        self.qslMessageTextEdit.setMaximumSize(QtCore.QSize(16777215, 50))
        self.qslMessageTextEdit.setTextInteractionFlags(QtCore.Qt.TextInteractionFlag.LinksAccessibleByKeyboard|QtCore.Qt.TextInteractionFlag.LinksAccessibleByMouse|QtCore.Qt.TextInteractionFlag.TextBrowserInteraction|QtCore.Qt.TextInteractionFlag.TextEditable|QtCore.Qt.TextInteractionFlag.TextEditorInteraction|QtCore.Qt.TextInteractionFlag.TextSelectableByKeyboard|QtCore.Qt.TextInteractionFlag.TextSelectableByMouse)
        self.qslMessageTextEdit.setProperty("cssClass", "QSOFormInput")
        self.qslMessageTextEdit.setObjectName("qslMessageTextEdit")
        self.verticalLayout_6.addWidget(self.qslMessageTextEdit)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.label_5 = QtWidgets.QLabel(parent=self.qslPage)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_12.addWidget(self.label_5)
        self.qslAccBureauCheckBox = QtWidgets.QCheckBox(parent=self.qslPage)
        self.qslAccBureauCheckBox.setObjectName("qslAccBureauCheckBox")
        self.horizontalLayout_12.addWidget(self.qslAccBureauCheckBox)
        self.qslAccDirectCheckBox = QtWidgets.QCheckBox(parent=self.qslPage)
        self.qslAccDirectCheckBox.setObjectName("qslAccDirectCheckBox")
        self.horizontalLayout_12.addWidget(self.qslAccDirectCheckBox)
        self.qslAcceQSLCheckBox = QtWidgets.QCheckBox(parent=self.qslPage)
        self.qslAcceQSLCheckBox.setObjectName("qslAcceQSLCheckBox")
        self.horizontalLayout_12.addWidget(self.qslAcceQSLCheckBox)
        self.qslAccLoTWCheckBox = QtWidgets.QCheckBox(parent=self.qslPage)
        self.qslAccLoTWCheckBox.setObjectName("qslAccLoTWCheckBox")
        self.horizontalLayout_12.addWidget(self.qslAccLoTWCheckBox)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem8)
        self.verticalLayout_6.addLayout(self.horizontalLayout_12)
        self.qslBurDirGroupBox = QtWidgets.QGroupBox(parent=self.qslPage)
        self.qslBurDirGroupBox.setCheckable(True)
        self.qslBurDirGroupBox.setChecked(False)
        self.qslBurDirGroupBox.setObjectName("qslBurDirGroupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.qslBurDirGroupBox)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.qslBureauRadioButton = QtWidgets.QRadioButton(parent=self.qslBurDirGroupBox)
        self.qslBureauRadioButton.setChecked(True)
        self.qslBureauRadioButton.setObjectName("qslBureauRadioButton")
        self.gridLayout_4.addWidget(self.qslBureauRadioButton, 0, 0, 1, 1)
        self.qslDirectRadioButton = QtWidgets.QRadioButton(parent=self.qslBurDirGroupBox)
        self.qslDirectRadioButton.setObjectName("qslDirectRadioButton")
        self.gridLayout_4.addWidget(self.qslDirectRadioButton, 0, 1, 1, 1)
        self.qslSentCheckBox = QtWidgets.QCheckBox(parent=self.qslBurDirGroupBox)
        self.qslSentCheckBox.setObjectName("qslSentCheckBox")
        self.gridLayout_4.addWidget(self.qslSentCheckBox, 1, 0, 1, 1)
        self.qslRcvdCheckBox = QtWidgets.QCheckBox(parent=self.qslBurDirGroupBox)
        self.qslRcvdCheckBox.setObjectName("qslRcvdCheckBox")
        self.gridLayout_4.addWidget(self.qslRcvdCheckBox, 1, 1, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout_4, 1, 0, 1, 1)
        spacerItem9 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_2.addItem(spacerItem9, 1, 1, 1, 1)
        self.verticalLayout_6.addWidget(self.qslBurDirGroupBox)
        spacerItem10 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        self.verticalLayout_6.addItem(spacerItem10)
        self.toolBox.addItem(self.qslPage, "")
        self.contestPage = QtWidgets.QWidget()
        self.contestPage.setGeometry(QtCore.QRect(0, 0, 432, 334))
        self.contestPage.setObjectName("contestPage")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.contestPage)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.contestGroupBox = QtWidgets.QGroupBox(parent=self.contestPage)
        self.contestGroupBox.setObjectName("contestGroupBox")
        self.formLayout_2 = QtWidgets.QFormLayout(self.contestGroupBox)
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_10 = QtWidgets.QLabel(parent=self.contestGroupBox)
        self.label_10.setObjectName("label_10")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_10)
        self.contestComboBox = QtWidgets.QComboBox(parent=self.contestGroupBox)
        self.contestComboBox.setEditable(True)
        self.contestComboBox.setProperty("cssClass", "QSOFormInput")
        self.contestComboBox.setObjectName("contestComboBox")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.contestComboBox)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.sentQSOSpinBox = QtWidgets.QSpinBox(parent=self.contestGroupBox)
        self.sentQSOSpinBox.setMinimum(0)
        self.sentQSOSpinBox.setMaximum(999)
        self.sentQSOSpinBox.setProperty("value", 0)
        self.sentQSOSpinBox.setProperty("cssClass", "QSOFormInput")
        self.sentQSOSpinBox.setObjectName("sentQSOSpinBox")
        self.horizontalLayout.addWidget(self.sentQSOSpinBox)
        spacerItem11 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem11)
        self.formLayout_2.setLayout(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.horizontalLayout)
        self.label_6 = QtWidgets.QLabel(parent=self.contestGroupBox)
        self.label_6.setObjectName("label_6")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_6)
        self.label_9 = QtWidgets.QLabel(parent=self.contestGroupBox)
        self.label_9.setObjectName("label_9")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_9)
        self.rcvdDataLineEdit = QtWidgets.QLineEdit(parent=self.contestGroupBox)
        self.rcvdDataLineEdit.setProperty("cssClass", "QSOFormInput")
        self.rcvdDataLineEdit.setObjectName("rcvdDataLineEdit")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.rcvdDataLineEdit)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.rcvdQSOSpinBox = QtWidgets.QSpinBox(parent=self.contestGroupBox)
        self.rcvdQSOSpinBox.setMinimum(0)
        self.rcvdQSOSpinBox.setMaximum(999)
        self.rcvdQSOSpinBox.setProperty("value", 0)
        self.rcvdQSOSpinBox.setProperty("cssClass", "QSOFormInput")
        self.rcvdQSOSpinBox.setObjectName("rcvdQSOSpinBox")
        self.horizontalLayout_4.addWidget(self.rcvdQSOSpinBox)
        spacerItem12 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem12)
        self.formLayout_2.setLayout(3, QtWidgets.QFormLayout.ItemRole.FieldRole, self.horizontalLayout_4)
        self.label_8 = QtWidgets.QLabel(parent=self.contestGroupBox)
        self.label_8.setObjectName("label_8")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_8)
        self.verticalLayout_9.addWidget(self.contestGroupBox)
        self.xotaGroupBox = QtWidgets.QGroupBox(parent=self.contestPage)
        self.xotaGroupBox.setObjectName("xotaGroupBox")
        self.formLayout = QtWidgets.QFormLayout(self.xotaGroupBox)
        self.formLayout.setObjectName("formLayout")
        self.eventComboBox = QtWidgets.QComboBox(parent=self.xotaGroupBox)
        self.eventComboBox.setProperty("cssClass", "QSOFormInput")
        self.eventComboBox.setObjectName("eventComboBox")
        self.eventComboBox.addItem("")
        self.eventComboBox.setItemText(0, "")
        self.eventComboBox.addItem("")
        self.eventComboBox.addItem("")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.eventComboBox)
        self.label_7 = QtWidgets.QLabel(parent=self.xotaGroupBox)
        self.label_7.setObjectName("label_7")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_7)
        self.label_11 = QtWidgets.QLabel(parent=self.xotaGroupBox)
        self.label_11.setObjectName("label_11")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_11)
        self.txExchLineEdit = QtWidgets.QLineEdit(parent=self.xotaGroupBox)
        self.txExchLineEdit.setProperty("cssClass", "QSOFormInput")
        self.txExchLineEdit.setObjectName("txExchLineEdit")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.txExchLineEdit)
        self.label_12 = QtWidgets.QLabel(parent=self.xotaGroupBox)
        self.label_12.setObjectName("label_12")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_12)
        self.rxExchLineEdit = QtWidgets.QLineEdit(parent=self.xotaGroupBox)
        self.rxExchLineEdit.setProperty("cssClass", "QSOFormInput")
        self.rxExchLineEdit.setObjectName("rxExchLineEdit")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.rxExchLineEdit)
        self.verticalLayout_9.addWidget(self.xotaGroupBox)
        spacerItem13 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        self.verticalLayout_9.addItem(spacerItem13)
        self.toolBox.addItem(self.contestPage, "")
        self.verticalLayout.addWidget(self.toolBox)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem14 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem14)
        self.savePushButton = QtWidgets.QPushButton(parent=QSOForm)
        self.savePushButton.setObjectName("savePushButton")
        self.horizontalLayout_2.addWidget(self.savePushButton)
        self.cancelPushButton = QtWidgets.QPushButton(parent=QSOForm)
        self.cancelPushButton.setObjectName("cancelPushButton")
        self.horizontalLayout_2.addWidget(self.cancelPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.retranslateUi(QSOForm)
        self.toolBox.setCurrentIndex(0)
        self.savePushButton.clicked.connect(QSOForm.saveLog) # type: ignore
        self.autoDateCheckBox.toggled['bool'].connect(self.dateOffEdit.setDisabled) # type: ignore
        self.bandComboBox.currentTextChanged['QString'].connect(QSOForm.bandChanged) # type: ignore
        self.stationGroupBox.toggled['bool'].connect(QSOForm.stationChanged) # type: ignore
        self.identityGroupBox.toggled['bool'].connect(QSOForm.identityChanged) # type: ignore
        self.identityGroupBox.toggled['bool'].connect(self.ownCallSignLineEdit.setDisabled) # type: ignore
        self.identityGroupBox.toggled['bool'].connect(self.ownNameLineEdit.setDisabled) # type: ignore
        self.channelComboBox.currentTextChanged['QString'].connect(QSOForm.channelChanged) # type: ignore
        self.RSTRcvdLineEdit.textEdited['QString'].connect(QSOForm.rstRcvdChanged) # type: ignore
        self.RSTSentLineEdit.textEdited['QString'].connect(QSOForm.rstSentChanged) # type: ignore
        self.locatorLineEdit.textEdited['QString'].connect(QSOForm.locatorChanged) # type: ignore
        self.callSignLineEdit.textEdited['QString'].connect(QSOForm.callSignChanged) # type: ignore
        self.ownCallSignLineEdit.textEdited['QString'].connect(QSOForm.ownCallSignChanged) # type: ignore
        self.callSignLineEdit.editingFinished.connect(QSOForm.callEditingFinished) # type: ignore
        self.timeNowPushButton.clicked.connect(QSOForm.setStartTimeNow) # type: ignore
        self.searchHamQTHPushButton.clicked.connect(QSOForm.searchHamQTH) # type: ignore
        self.cancelPushButton.clicked.connect(QSOForm.clear) # type: ignore
        self.modeComboBox.currentTextChanged['QString'].connect(QSOForm.modeChanged) # type: ignore
        self.autoDateCheckBox.toggled['bool'].connect(self.timeOffEdit.setDisabled) # type: ignore
        self.timeOnEdit.textEdited['QString'].connect(QSOForm.timeOnChanged) # type: ignore
        self.timeOffEdit.textEdited['QString'].connect(QSOForm.timeOffChanged) # type: ignore
        self.autoDateCheckBox.toggled['bool'].connect(QSOForm.autoDateCBChanged) # type: ignore
        self.searchQRZCQPushButton.clicked.connect(QSOForm.searchQRZCQ) # type: ignore
        self.stationGroupBox.toggled['bool'].connect(self.antennaComboBox.setDisabled) # type: ignore
        self.stationGroupBox.toggled['bool'].connect(self.radioComboBox.setDisabled) # type: ignore
        self.stationGroupBox.toggled['bool'].connect(self.ownQTHComboBox.setDisabled) # type: ignore
        self.ownQTHComboBox.editTextChanged['QString'].connect(QSOForm.ownQTHChanged) # type: ignore
        self.contestComboBox.currentTextChanged['QString'].connect(QSOForm.contestChanged) # type: ignore
        self.eventComboBox.currentTextChanged['QString'].connect(QSOForm.eventChanged) # type: ignore
        self.searchQRZPushButton.clicked.connect(QSOForm.searchQRZ) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(QSOForm)
        QSOForm.setTabOrder(self.callSignLineEdit, self.nameLineEdit)
        QSOForm.setTabOrder(self.nameLineEdit, self.QTHLineEdit)
        QSOForm.setTabOrder(self.QTHLineEdit, self.locatorLineEdit)
        QSOForm.setTabOrder(self.locatorLineEdit, self.RSTSentLineEdit)
        QSOForm.setTabOrder(self.RSTSentLineEdit, self.RSTRcvdLineEdit)
        QSOForm.setTabOrder(self.RSTRcvdLineEdit, self.powerSpinBox)
        QSOForm.setTabOrder(self.powerSpinBox, self.bandComboBox)
        QSOForm.setTabOrder(self.bandComboBox, self.channelComboBox)
        QSOForm.setTabOrder(self.channelComboBox, self.freqDoubleSpinBox)
        QSOForm.setTabOrder(self.freqDoubleSpinBox, self.modeComboBox)
        QSOForm.setTabOrder(self.modeComboBox, self.submodeComboBox)
        QSOForm.setTabOrder(self.submodeComboBox, self.propComboBox)
        QSOForm.setTabOrder(self.propComboBox, self.dateOnEdit)
        QSOForm.setTabOrder(self.dateOnEdit, self.timeOnEdit)
        QSOForm.setTabOrder(self.timeOnEdit, self.timeNowPushButton)
        QSOForm.setTabOrder(self.timeNowPushButton, self.dateOffEdit)
        QSOForm.setTabOrder(self.dateOffEdit, self.timeOffEdit)
        QSOForm.setTabOrder(self.timeOffEdit, self.autoDateCheckBox)
        QSOForm.setTabOrder(self.autoDateCheckBox, self.commentLineEdit)
        QSOForm.setTabOrder(self.commentLineEdit, self.savePushButton)
        QSOForm.setTabOrder(self.savePushButton, self.cancelPushButton)
        QSOForm.setTabOrder(self.cancelPushButton, self.identityGroupBox)
        QSOForm.setTabOrder(self.identityGroupBox, self.ownCallSignLineEdit)
        QSOForm.setTabOrder(self.ownCallSignLineEdit, self.ownNameLineEdit)
        QSOForm.setTabOrder(self.ownNameLineEdit, self.stationGroupBox)
        QSOForm.setTabOrder(self.stationGroupBox, self.ownQTHComboBox)
        QSOForm.setTabOrder(self.ownQTHComboBox, self.radioComboBox)
        QSOForm.setTabOrder(self.radioComboBox, self.antennaComboBox)
        QSOForm.setTabOrder(self.antennaComboBox, self.remarksTextEdit)
        QSOForm.setTabOrder(self.remarksTextEdit, self.qslViaLineEdit)
        QSOForm.setTabOrder(self.qslViaLineEdit, self.qslMessageTextEdit)
        QSOForm.setTabOrder(self.qslMessageTextEdit, self.qslAccBureauCheckBox)
        QSOForm.setTabOrder(self.qslAccBureauCheckBox, self.qslAccDirectCheckBox)
        QSOForm.setTabOrder(self.qslAccDirectCheckBox, self.qslAcceQSLCheckBox)
        QSOForm.setTabOrder(self.qslAcceQSLCheckBox, self.qslAccLoTWCheckBox)
        QSOForm.setTabOrder(self.qslAccLoTWCheckBox, self.qslBurDirGroupBox)
        QSOForm.setTabOrder(self.qslBurDirGroupBox, self.qslBureauRadioButton)
        QSOForm.setTabOrder(self.qslBureauRadioButton, self.qslDirectRadioButton)
        QSOForm.setTabOrder(self.qslDirectRadioButton, self.qslSentCheckBox)
        QSOForm.setTabOrder(self.qslSentCheckBox, self.qslRcvdCheckBox)
        QSOForm.setTabOrder(self.qslRcvdCheckBox, self.contestComboBox)
        QSOForm.setTabOrder(self.contestComboBox, self.sentQSOSpinBox)
        QSOForm.setTabOrder(self.sentQSOSpinBox, self.rcvdDataLineEdit)
        QSOForm.setTabOrder(self.rcvdDataLineEdit, self.rcvdQSOSpinBox)
        QSOForm.setTabOrder(self.rcvdQSOSpinBox, self.eventComboBox)
        QSOForm.setTabOrder(self.eventComboBox, self.txExchLineEdit)
        QSOForm.setTabOrder(self.txExchLineEdit, self.rxExchLineEdit)

    def retranslateUi(self, QSOForm):
        _translate = QtCore.QCoreApplication.translate
        QSOForm.setWindowTitle(_translate("QSOForm", "QSO Form"))
        self.nameLineEdit.setPlaceholderText(_translate("QSOForm", "Name"))
        self.locatorLineEdit.setPlaceholderText(_translate("QSOForm", "Locator"))
        self.callSignLineEdit.setPlaceholderText(_translate("QSOForm", "Call sign"))
        self.QTHLineEdit.setPlaceholderText(_translate("QSOForm", "QTH"))
        self.searchHamQTHPushButton.setToolTip(_translate("QSOForm", "Lookup at HamQTH.com (Ctrl+Alt+H)"))
        self.searchHamQTHPushButton.setText(_translate("QSOForm", "HamQTH"))
        self.searchHamQTHPushButton.setShortcut(_translate("QSOForm", "Ctrl+Alt+H"))
        self.searchQRZPushButton.setToolTip(_translate("QSOForm", "Lookup at QRZ.com (Ctrl+Shift+Q)"))
        self.searchQRZPushButton.setText(_translate("QSOForm", "QRZ"))
        self.searchQRZPushButton.setShortcut(_translate("QSOForm", "Ctrl+Shift+Q"))
        self.searchQRZCQPushButton.setToolTip(_translate("QSOForm", "Lookup at QRZCQ.com (Ctrl+Alt+Z)"))
        self.searchQRZCQPushButton.setText(_translate("QSOForm", "QRZCQ"))
        self.searchQRZCQPushButton.setShortcut(_translate("QSOForm", "Ctrl+Alt+Z"))
        self.label.setText(_translate("QSOForm", "RST Tx"))
        self.RSTSentLineEdit.setText(_translate("QSOForm", "59"))
        self.RSTSentLineEdit.setPlaceholderText(_translate("QSOForm", "RST sent"))
        self.label_2.setText(_translate("QSOForm", "RST Rx"))
        self.RSTRcvdLineEdit.setText(_translate("QSOForm", "59"))
        self.RSTRcvdLineEdit.setPlaceholderText(_translate("QSOForm", "RST received"))
        self.powerLabel.setText(_translate("QSOForm", "Power"))
        self.powerSpinBox.setSpecialValueText(_translate("QSOForm", "n.a."))
        self.powerSpinBox.setSuffix(_translate("QSOForm", " W"))
        self.bandComboBox.setPlaceholderText(_translate("QSOForm", "Band"))
        self.channelLabel.setText(_translate("QSOForm", "Channel"))
        self.freqLabel.setText(_translate("QSOForm", "Frequency"))
        self.freqDoubleSpinBox.setSpecialValueText(_translate("QSOForm", "n.a."))
        self.freqDoubleSpinBox.setSuffix(_translate("QSOForm", " kHz"))
        self.modeComboBox.setPlaceholderText(_translate("QSOForm", "Mode"))
        self.submodeComboBox.setPlaceholderText(_translate("QSOForm", "Submode"))
        self.propComboBox.setPlaceholderText(_translate("QSOForm", "Propagation"))
        self.timeNowPushButton.setText(_translate("QSOForm", "Now"))
        self.timeNowPushButton.setShortcut(_translate("QSOForm", "Ctrl+Alt+N"))
        self.label_3.setText(_translate("QSOForm", "Start"))
        self.timeOffEdit.setPlaceholderText(_translate("QSOForm", "Time end"))
        self.dateOnEdit.setDisplayFormat(_translate("QSOForm", "yyyy-MM-dd"))
        self.label_4.setText(_translate("QSOForm", "End"))
        self.dateOffEdit.setDisplayFormat(_translate("QSOForm", "yyyy-MM-dd"))
        self.autoDateCheckBox.setText(_translate("QSOForm", "Automatically"))
        self.timeOnEdit.setPlaceholderText(_translate("QSOForm", "Time start"))
        self.commentLineEdit.setPlaceholderText(_translate("QSOForm", "QSO comment"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.mainPage), _translate("QSOForm", "Main data"))
        self.identityGroupBox.setTitle(_translate("QSOForm", "Configured identity"))
        self.ownCallSignLineEdit.setPlaceholderText(_translate("QSOForm", "Own call sign"))
        self.ownNameLineEdit.setPlaceholderText(_translate("QSOForm", "Own name"))
        self.stationGroupBox.setTitle(_translate("QSOForm", "Configured station"))
        self.remarksTextEdit.setPlaceholderText(_translate("QSOForm", "Own notes"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.stationPage), _translate("QSOForm", "Own data"))
        self.qslViaLineEdit.setToolTip(_translate("QSOForm", "The contacted station QSL route.\n"
"This is not the QSL manager/bureau address."))
        self.qslViaLineEdit.setPlaceholderText(_translate("QSOForm", "QSL via"))
        self.qslMessageTextEdit.setPlaceholderText(_translate("QSOForm", "QSL card message"))
        self.label_5.setText(_translate("QSOForm", "Accepts:"))
        self.qslAccBureauCheckBox.setText(_translate("QSOForm", "Bureau"))
        self.qslAccDirectCheckBox.setText(_translate("QSOForm", "Direct"))
        self.qslAcceQSLCheckBox.setText(_translate("QSOForm", "eQSL"))
        self.qslAccLoTWCheckBox.setText(_translate("QSOForm", "LoTW"))
        self.qslBurDirGroupBox.setTitle(_translate("QSOForm", "Bureau/Direct"))
        self.qslBureauRadioButton.setText(_translate("QSOForm", "Bureau"))
        self.qslDirectRadioButton.setText(_translate("QSOForm", "Direct"))
        self.qslSentCheckBox.setText(_translate("QSOForm", "QSL sent"))
        self.qslRcvdCheckBox.setText(_translate("QSOForm", "QSL received"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.qslPage), _translate("QSOForm", "QSL"))
        self.contestGroupBox.setTitle(_translate("QSOForm", "Contest"))
        self.label_10.setText(_translate("QSOForm", "Contest ID"))
        self.contestComboBox.setPlaceholderText(_translate("QSOForm", "Contest ID"))
        self.sentQSOSpinBox.setSpecialValueText(_translate("QSOForm", "n.a."))
        self.label_6.setText(_translate("QSOForm", "Sent QSO ID"))
        self.label_9.setText(_translate("QSOForm", "Rx Exchange"))
        self.rcvdDataLineEdit.setPlaceholderText(_translate("QSOForm", "Rx Exchange"))
        self.rcvdQSOSpinBox.setSpecialValueText(_translate("QSOForm", "n.a."))
        self.label_8.setText(_translate("QSOForm", "Rcvd QSO ID"))
        self.xotaGroupBox.setTitle(_translate("QSOForm", "xOTA"))
        self.eventComboBox.setItemText(1, _translate("QSOForm", "POTA"))
        self.eventComboBox.setItemText(2, _translate("QSOForm", "SOTA"))
        self.label_7.setText(_translate("QSOForm", "Event"))
        self.label_11.setText(_translate("QSOForm", "Tx Exchange"))
        self.txExchLineEdit.setPlaceholderText(_translate("QSOForm", "Tx Exchange"))
        self.label_12.setText(_translate("QSOForm", "Rx Exchange"))
        self.rxExchLineEdit.setPlaceholderText(_translate("QSOForm", "Rx Exchange"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.contestPage), _translate("QSOForm", "Contest && xOTA"))
        self.savePushButton.setText(_translate("QSOForm", "Save"))
        self.savePushButton.setShortcut(_translate("QSOForm", "Ctrl+S"))
        self.cancelPushButton.setText(_translate("QSOForm", "Clear"))



# DragonLog (c) 2023-2025 by Andreas Schawo is licensed under CC BY-SA 4.0.
# To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/

import os
import logging

from PyQt6 import QtWidgets, QtCore

from . import ContestDlg_ui
from .Logger import Logger
from .contest import CONTESTS, CONTEST_IDS, CONTEST_NAMES
from .contest.base import (ContestLog, ContestLogEDI, Address,
                           CategoryBand, CategoryMode, CategoryPower, CategoryOperator)
from . import ColorPalettes
from .RegEx import check_qth, check_format, find_non_ascii, REGEX_CALL, REGEX_LOCATOR, REGEX_EMAIL
from .cty import CountryData


class ContestDialog(QtWidgets.QDialog, ContestDlg_ui.Ui_ContestDialog):
    def __init__(self, parent, dragonlog, settings: QtCore.QSettings, logger: Logger, contests: list,
                 cty: CountryData):
        super().__init__(parent)
        self.dragonlog = dragonlog
        self.setupUi(self)

        self.log = logging.getLogger('ContestDialog')
        self.log.addHandler(logger)
        self.log.setLevel(logger.loglevel)
        self.logger = logger
        self.log.debug('Initialising...')

        self.__cty__ = cty

        self.__settings__ = settings

        self.contest: type[ContestLog] | None = None
        self.contestComboBox.insertItems(0, [CONTEST_NAMES[c] for c in contests])
        if CONTEST_IDS.get(self.__settings__.value('contest/id', ''), '') in contests:
            self.contestComboBox.setCurrentText(self.__settings__.value('contest/id', ''))

        self.__validation__: dict[str, bool] = {}

        cur_date = QtCore.QDateTime.currentDateTimeUtc().toString('yyyy-MM-dd')
        date_from = self.__settings__.value('contest/from_date', cur_date)
        self.fromDateEdit.setDate(QtCore.QDate.fromString(date_from, 'yyyy-MM-dd'))
        date_to = self.__settings__.value('contest/to_date', cur_date)
        self.toDateEdit.setDate(QtCore.QDate.fromString(date_to, 'yyyy-MM-dd'))

        city = ''
        locator = ''
        if check_qth(settings.value('station/qth_loc', '')):
            city, locator = check_qth(settings.value('station/qth_loc', ''))

        self.callLineEdit.setText(self.__settings__.value('contest/call', settings.value('station/callSign', '')))
        self.callChanged(self.callLineEdit.text())
        self.operatorsLineEdit.setText(self.callLineEdit.text())
        self.locatorLineEdit.setText(self.__settings__.value('contest/locator', locator))
        self.locatorChanged(self.locatorLineEdit.text())
        self.nameLineEdit.setText(self.__settings__.value('contest/name', settings.value('station/name', '')))
        self.nameChanged(self.nameLineEdit.text())
        self.emailLineEdit.setText(self.__settings__.value('contest/email', ''))
        self.emailChanged(self.emailLineEdit.text())
        self.streetLineEdit.setText(self.__settings__.value('contest/street', ''))
        self.streetChanged(self.streetLineEdit.text())
        self.zipLineEdit.setText(self.__settings__.value('contest/zip', ''))
        self.zipChanged(self.zipLineEdit.text())
        self.cityLineEdit.setText(self.__settings__.value('contest/city', ''))
        self.cityChanged(self.cityLineEdit.text())
        self.countryLineEdit.setText(self.__settings__.value('contest/country', ''))
        self.countryChanged(self.countryLineEdit.text())
        self.specificLineEdit.setText(self.__settings__.value('contest/specific', ''))
        self.specificChanged(self.specificLineEdit.text())
        self.clubLineEdit.setText(self.__settings__.value('contest/club', ''))
        self.clubChanged(self.clubLineEdit.text())

        self.qthLineEdit.setText(city)
        self.rigLineEdit.setText(self.__settings__.value('station/radio', ''))
        self.powerSpinBox.setValue(int(self.__settings__.value('contest_edi/power', 0)))
        self.antennaLineEdit.setText(self.__settings__.value('station/antenna', ''))
        self.antAboveGroundSpinBox.setValue(int(self.__settings__.value('contest_edi/ant_height_ground', -1)))
        self.antAboveSeaSpinBox.setValue(int(self.__settings__.value('contest_edi/ant_height_sea', -1)))

        self.exportPathLineEdit.setText(self.__settings__.value('contest/last_export_dir', os.path.abspath(os.curdir)))

    def contestChanged(self, contest: str):
        self.contest = CONTESTS[CONTEST_IDS[contest]]

        self.bandComboBox.clear()
        self.bandComboBox.insertItems(0, self.contest.valid_bands_list())

        self.toDateEdit.setEnabled(not self.contest.is_single_day())
        self.toDateEdit.setMinimumDate(self.fromDateEdit.date())
        self.toDateEdit.setDate(self.fromDateEdit.date())

        self.specificLabel.setText(self.contest.descr_specific())

        self.modeComboBox.clear()
        self.modeComboBox.insertItems(0, self.contest.valid_modes_list())

        self.powerComboBox.clear()
        self.powerComboBox.insertItems(0, self.contest.valid_power_list())

        self.opComboBox.clear()
        self.opComboBox.insertItems(0, self.contest.valid_operator_list())

        self.ediGroupBox.setEnabled(issubclass(self.contest, ContestLogEDI))

    def fromDateChanged(self, date: QtCore.QDate):
        if self.contest.is_single_day():
            self.toDateEdit.setMinimumDate(date)
            self.toDateEdit.setDate(date)
        else:
            self.toDateEdit.setMinimumDate(date.addDays(1))

    def choosePath(self):
        res = QtWidgets.QFileDialog.getExistingDirectory(
            self,
            self.tr('Select export folder'),
            self.__settings__.value('contest/last_export_dir', os.path.abspath(os.curdir)))

        if res:
            self.exportPathLineEdit.setText(res)
            self.__settings__.setValue('contest/last_export_dir', res)

    def checkRequired(self):
        if all(self.__validation__.values()):
            self.exportPushButton.setEnabled(True)
        else:
            self.exportPushButton.setEnabled(False)

    def callChanged(self, txt):
        if not txt.strip():
            self.callLineEdit.setPalette(ColorPalettes.PaletteRequired)
            self.__validation__['call'] = False
        elif check_format(REGEX_CALL, txt):
            self.callLineEdit.setPalette(ColorPalettes.PaletteOk)
            self.__validation__['call'] = True
        else:
            self.callLineEdit.setPalette(ColorPalettes.PaletteFaulty)
            self.__validation__['call'] = True

        self.checkRequired()

    def locatorChanged(self, txt):
        if not txt.strip():
            self.locatorLineEdit.setPalette(ColorPalettes.PaletteRequired)
            self.__validation__['locator'] = False
        elif check_format(REGEX_LOCATOR, txt):
            self.locatorLineEdit.setPalette(ColorPalettes.PaletteOk)
            self.__validation__['locator'] = True
        else:
            self.locatorLineEdit.setPalette(ColorPalettes.PaletteFaulty)
            self.__validation__['locator'] = True

        self.checkRequired()

    def nameChanged(self, txt):
        if not txt.strip():
            self.nameLineEdit.setPalette(ColorPalettes.PaletteRequired)
            self.__validation__['name'] = False
        elif ' ' in txt.strip() and not find_non_ascii(txt):
            self.nameLineEdit.setPalette(ColorPalettes.PaletteOk)
            self.__validation__['name'] = True
        else:
            self.nameLineEdit.setPalette(ColorPalettes.PaletteFaulty)
            self.__validation__['name'] = True

        self.checkRequired()

    def emailChanged(self, txt):
        if not txt.strip():
            self.emailLineEdit.setPalette(ColorPalettes.PaletteRequired)
            self.__validation__['email'] = False
        elif check_format(REGEX_EMAIL, txt):
            self.emailLineEdit.setPalette(ColorPalettes.PaletteOk)
            self.__validation__['email'] = True
        else:
            self.emailLineEdit.setPalette(ColorPalettes.PaletteFaulty)
            self.__validation__['email'] = True

        self.checkRequired()

    def streetChanged(self, txt):
        if find_non_ascii(txt):
            self.streetLineEdit.setPalette(ColorPalettes.PaletteFaulty)
        else:
            self.streetLineEdit.setPalette(ColorPalettes.PaletteDefault)

    def zipChanged(self, txt):
        if find_non_ascii(txt):
            self.zipLineEdit.setPalette(ColorPalettes.PaletteFaulty)
        else:
            self.zipLineEdit.setPalette(ColorPalettes.PaletteDefault)

    def cityChanged(self, txt):
        if find_non_ascii(txt):
            self.cityLineEdit.setPalette(ColorPalettes.PaletteFaulty)
        else:
            self.cityLineEdit.setPalette(ColorPalettes.PaletteDefault)

    def countryChanged(self, txt):
        if find_non_ascii(txt):
            self.countryLineEdit.setPalette(ColorPalettes.PaletteFaulty)
        else:
            self.countryLineEdit.setPalette(ColorPalettes.PaletteDefault)

    def specificChanged(self, txt):
        if not txt.strip() and self.contest.needs_specific():
            self.specificLineEdit.setPalette(ColorPalettes.PaletteRequired)
            self.__validation__['specific'] = False
        elif find_non_ascii(txt):
            self.specificLineEdit.setPalette(ColorPalettes.PaletteFaulty)
            self.__validation__['specific'] = True
        else:
            self.specificLineEdit.setPalette(ColorPalettes.PaletteOk)
            self.__validation__['specific'] = True

        self.checkRequired()

    def clubChanged(self, txt):
        if not txt.strip():
            self.clubLineEdit.setPalette(ColorPalettes.PaletteRequired)
            self.__validation__['club'] = False
        elif find_non_ascii(txt):
            self.clubLineEdit.setPalette(ColorPalettes.PaletteFaulty)
            self.__validation__['club'] = True
        else:
            self.clubLineEdit.setPalette(ColorPalettes.PaletteOk)
            self.__validation__['club'] = True

        self.checkRequired()

    def operatorChanged(self, op_class):
        if op_class in ('MULTI', 'TRAINEE'):
            self.operatorsLineEdit.setEnabled(True)
        else:
            self.operatorsLineEdit.setEnabled(False)
        self.operatorsLineEdit.setText(self.callLineEdit.text())

    def accept(self):
        self.log.info(f'Exporting "{self.contestComboBox.currentText()}" from {self.fromDateEdit.text()}...')

        c_filter = f"""SELECT * FROM qsos WHERE 
    contest_id = '{CONTEST_IDS[self.contestComboBox.currentText()]}' AND 
    DATE(date_time) >= DATE('{self.fromDateEdit.text()}') AND
    DATE(date_time) <= DATE('{self.toDateEdit.text()}') """

        if self.bandComboBox.currentText() != 'all':
            c_filter += f"AND band == '{self.bandComboBox.currentText()}' "

        c_filter += 'ORDER BY date_time,id'

        # noinspection PyProtectedMember
        doc = self.dragonlog._build_adif_export_(c_filter,
                                                 include_id=True)

        addr = Address(
            self.streetLineEdit.text(),
            self.zipLineEdit.text(),
            self.cityLineEdit.text(),
            self.countryLineEdit.text()
        )

        # noinspection PyTypeChecker
        contest: ContestLog = self.contest(self.__settings__.value('station/callSign', 'XX1XXX').upper(),
                                           self.nameLineEdit.text(),
                                           self.clubLineEdit.text(),
                                           addr,
                                           self.emailLineEdit.text(),
                                           self.locatorLineEdit.text(),
                                           CategoryBand.from_str(self.bandComboBox.currentText()),
                                           CategoryMode.from_str(self.modeComboBox.currentText()),
                                           CategoryPower.from_str(self.powerComboBox.currentText()),
                                           CategoryOperator.from_str(self.opComboBox.currentText()),
                                           operators=self.operatorsLineEdit.text().split(' '),
                                           specific=self.specificLineEdit.text(),
                                           logger=self.logger,
                                           # Extra parameters for edi format
                                           from_date=self.fromDateEdit.text(),
                                           to_date=self.toDateEdit.text(),
                                           qth=self.qthLineEdit.text(),
                                           radio=self.rigLineEdit.text(),
                                           pwr_watts=self.powerSpinBox.value(),
                                           antenna=self.antennaLineEdit.text(),
                                           ant_height_ground=self.antAboveGroundSpinBox.value(),
                                           ant_height_sea=self.antAboveSeaSpinBox.value(),
                                           # Extra parameters for some contests
                                           cty=self.__cty__,
                                           )
        contest.set_created_by(f'{self.dragonlog.programName} {self.dragonlog.programVersion}')

        if self.soapPlainTextEdit.toPlainText().strip():
            contest.add_soapbox(self.soapPlainTextEdit.toPlainText().strip())

        if doc['RECORDS']:
            try:
                for rec in doc['RECORDS']:
                    contest.append(rec)

                self.log.info(f'Contest statistics {contest.summary()}')
                contest.open_file(self.exportPathLineEdit.text())
                contest.write_records()
                contest.close_file()
                self.log.info(f'Records written to "{contest.file_name}"')

                if contest.errors or contest.warnings:
                    QtWidgets.QMessageBox.information(self, self.tr('Contest Export'),
                                                      self.tr('There were {} error(s) and {} warning(s) '
                                                              'when processing the contest data').format(contest.errors,
                                                                                                         contest.warnings) +
                                                      '\n\n' +
                                                      self.tr(
                                                          'Export will be written anyway. '
                                                          'Please check the application log before sending!')
                                                      )

                QtWidgets.QMessageBox.information(self, self.tr('Contest Export'),
                                                  self.tr('Contest data written to') + f' {contest.file_name}\n' +
                                                  self.tr('Claimed points') + f': {contest.claimed_points}\n\n' +
                                                  self.tr('Please check the file patiently before sending!')
                                                  )
            except Exception as exc:
                self.log.exception(exc)
                QtWidgets.QMessageBox.critical(self, self.tr('Contest Export'),
                                               self.tr(
                                                   'Error processing contest data.\n'
                                                   'Please check log for further information.'))
        else:
            QtWidgets.QMessageBox.warning(self, self.tr('Contest Export'),
                                          self.tr('No contest data found for export') +
                                          f'\n"{self.contestComboBox.currentText()}" from {self.fromDateEdit.text()}')

        self.__settings__.setValue('contest/id', self.contestComboBox.currentText())
        self.__settings__.setValue('contest/from_date', self.fromDateEdit.text())
        self.__settings__.setValue('contest/to_date', self.toDateEdit.text())
        self.__settings__.setValue('contest/call', self.callLineEdit.text())
        self.__settings__.setValue('contest/locator', self.locatorLineEdit.text())
        self.__settings__.setValue('contest/name', self.nameLineEdit.text())
        self.__settings__.setValue('contest/email', self.emailLineEdit.text())
        self.__settings__.setValue('contest/street', self.streetLineEdit.text())
        self.__settings__.setValue('contest/zip', self.zipLineEdit.text())
        self.__settings__.setValue('contest/city', self.cityLineEdit.text())
        self.__settings__.setValue('contest/country', self.countryLineEdit.text())
        self.__settings__.setValue('contest/specific', self.specificLineEdit.text())
        self.__settings__.setValue('contest/club', self.clubLineEdit.text())

        self.__settings__.setValue('contest_edi/power', self.powerSpinBox.value())
        self.__settings__.setValue('contest_edi/ant_height_ground', self.antAboveGroundSpinBox.value())
        self.__settings__.setValue('contest_edi/ant_height_sea', self.antAboveSeaSpinBox.value())

        super().accept()



# DragonLog (c) 2025 by Andreas Schawo is licensed under CC BY-SA 4.0.
# To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/
"""A local callbook for ham radio callsigns and associated data and contest call history"""

import logging
import sys
import csv
import json
import sqlite3
import datetime
from collections import namedtuple
from dataclasses import dataclass, fields, astuple, asdict
from enum import Enum, auto

from dragonlog.RegEx import check_call


def get_cur_dt() -> str:
    """Get current date/time independant of Python version"""
    if sys.version_info[0] == 3 and sys.version_info[1] < 11:
        # noinspection PyDeprecation
        return datetime.datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S')

    return datetime.datetime.now(datetime.UTC).strftime('%Y-%m-%d %H:%M:%S')


# Local Callbook
@dataclass
class LocalCallbookData:
    """Callbook entry for a callsign"""
    name: str = ''
    locator: str = ''
    qth: str = ''
    qsl_via: str = ''
    qsl_bureau: str = 'N'
    qsl_direct: str = 'N'
    qsl_eqsl: str = 'N'
    qsl_lotw: str = 'N'
    darc_dok: str = ''


def adapt_callbook_data(cbd: LocalCallbookData) -> str:
    return json.dumps(asdict(cbd))


def convert_callbook_data(data: bytes) -> LocalCallbookData:
    data = json.loads(data.decode('utf8'))
    return LocalCallbookData(**data)


sqlite3.register_adapter(LocalCallbookData, adapt_callbook_data)
sqlite3.register_converter('CALLBOOKDATA', convert_callbook_data)


# Call history
@dataclass
class CallHistoryData:
    """Represents history data for a callsign"""
    locator: str = ''
    power_class: str = ''
    darc_dok: str = ''
    itu_zone: str = ''


def adapt_history_data(chd: CallHistoryData) -> str:
    return json.dumps(asdict(chd))


def convert_history_data(data: bytes) -> CallHistoryData:
    data = json.loads(data.decode('utf8'))
    return CallHistoryData(**data)


sqlite3.register_adapter(CallHistoryData, adapt_history_data)
sqlite3.register_converter('CALLHISTORYDATA', convert_history_data)


class LocalCallbookExportError(Exception):
    pass


class LocalCallbookImportError(Exception):
    pass


class LocalCallbookDatabaseError(Exception):
    pass


class UpdateMode(Enum):
    Complement = auto()
    Overwrite = auto()
    Substitute = auto()


LocalCallbookResult = namedtuple('LocalCallbookResult',
                                 ('callsign', 'callbook_data'))

CallHistoryResult = namedtuple('CallHistoryResult',
                               ('contest', 'callsign', 'history_data', 'callbook_data'))


class LocalCallbook:
    __db_create_stmnt__ = '''CREATE TABLE IF NOT EXISTS "callbook" (
                                "callsign"  TEXT PRIMARY KEY NOT NULL,
                                "recorded"  NUMERIC NOT NULL,
                                "data" CALLBOOKDATA NOT NULL
                             );'''

    __db_create_view_stmnt_count__ = '''CREATE VIEW IF NOT EXISTS callbook_entries AS 
                                     SELECT COUNT(callsign) as entries FROM callbook'''

    __db_create_stmnt_hist__ = '''CREATE TABLE IF NOT EXISTS "history" (
                                "contest"    TEXT NOT NULL,
                                "callsign"  TEXT NOT NULL,
                                "recorded"  NUMERIC NOT NULL,
                                "data" CALLHISTORYDATA
                                );'''

    __db_create_idx_stmnt_hist__ = '''CREATE INDEX IF NOT EXISTS "call_history" ON "history" (
                                    "contest",
                                    "callsign"
                                );'''

    __db_create_view_stmnt_hist_count__ = '''CREATE VIEW IF NOT EXISTS history_entries AS 
                                     SELECT COUNT(contest) as entries, COUNT(DISTINCT contest) as contests FROM history;'''

    __db_create_view_stmnt_hist__ = '''CREATE VIEW IF NOT EXISTS history_callbook AS
                                    SELECT contest, history.callsign, history.recorded, history.data as history_data, 
                                           callbook.data as callbook_data FROM history 
                                    LEFT JOIN callbook 
                                    ON history.callsign == callbook.callsign;'''

    def __init__(self, db_filename: str, logger=None, csv_delimiter: str = ','):
        self.log = logging.getLogger(type(self).__name__)
        if logger:
            self.log.setLevel(logger.loglevel)
            self.log.addHandler(logger)
        self.log.debug('Initialising...')

        self.__csv_delimiter__ = csv_delimiter
        self.__new_db__ = False
        try:
            self.__db__ = sqlite3.connect(db_filename, detect_types=sqlite3.PARSE_DECLTYPES)
            self.log.info(f'Using callbook "{db_filename}"')
            self.__db_path__ = db_filename
            self._init_db_()
        except sqlite3.DatabaseError as exc:
            raise LocalCallbookDatabaseError(str(exc))

    def _init_db_(self):
        self.__db__.execute('PRAGMA user_version = 0xDF1A5C;')
        self.__db__.execute('PRAGMA application_id = 0xca1b004;')
        self.__db__.execute('PRAGMA temp_store = MEMORY;')
        self.__db__.execute('PRAGMA journal_mode = WAL;')
        self.__db__.execute('PRAGMA synchronous = NORMAL;')

        cur = self.__db__.execute('SELECT GROUP_CONCAT(NAME,",") as columns FROM PRAGMA_TABLE_INFO("callbook")')
        if not cur.fetchone()[0]:
            self.__new_db__ = True
            self.log.debug('Initialising table "callbook"...')
            self.__db__.execute(self.__db_create_stmnt__)
        self.__db__.execute(self.__db_create_view_stmnt_count__)

        cur = self.__db__.execute('SELECT GROUP_CONCAT(NAME,",") as columns FROM PRAGMA_TABLE_INFO("history")')
        if not cur.fetchone()[0]:
            self.__new_db__ = True
            self.log.debug('Initialising table "history"...')
            self.__db__.execute(self.__db_create_stmnt_hist__)
        self.__db__.execute(self.__db_create_idx_stmnt_hist__)
        self.__db__.execute(self.__db_create_view_stmnt_hist_count__)
        self.__db__.execute(self.__db_create_view_stmnt_hist__)

    @property
    def callbook_entries(self) -> int:
        """The number of entries in the callbook"""
        cur = self.__db__.execute('SELECT * FROM callbook_entries')
        return cur.fetchone()[0]

    @property
    def history_entries(self) -> tuple[int, int]:
        """The number of entries and contests in the history"""
        cur = self.__db__.execute('SELECT * FROM history_entries')
        return cur.fetchone()

    @property
    def path(self) -> str:
        """The path to the database"""
        return self.__db_path__

    @property
    def is_new(self) -> bool:
        """The database is new and may need some callbook entries"""
        return self.__new_db__

    def import_callbook(self, filename: str):
        self.log.info(f'Importing callbook from CSV "{filename}"...')
        try:
            with open(filename, encoding='utf-8') as cf:
                reader = csv.DictReader(cf, delimiter=self.__csv_delimiter__)
                d_fields = [f.name for f in fields(LocalCallbookData)]
                for hf in reader.fieldnames:
                    if hf not in ('callsign', 'recorded') and hf not in d_fields:
                        self.log.warning(f'Unsupported field "{hf}" in callbook import')
                for row in reader:
                    callsign = row.pop('callsign')
                    row.pop('recorded', None)
                    for f in tuple(row.keys()):
                        if not f in d_fields:
                            row.pop(f)
                    if row:
                        self.update_entry(callsign, LocalCallbookData(**row), UpdateMode.Substitute)
        except KeyError as exc:
            raise LocalCallbookImportError(f'Column {exc} is missing in callbook import') from None
        except OSError as exc:
            raise LocalCallbookImportError(str(exc)) from None

    def export_callbook(self, filename: str):
        self.log.info(f'Exporting to CSV "{filename}"...')
        try:
            with open(filename, 'w', newline='', encoding='utf-8') as cf:
                writer = csv.writer(cf, delimiter=self.__csv_delimiter__)
                # Write header
                writer.writerow(['callsign', 'recorded'] + [f.name for f in fields(LocalCallbookData)])

                cur = self.__db__.execute('SELECT * FROM callbook '
                                          'ORDER BY callsign')
                row: tuple[str, str, LocalCallbookData]
                for row in cur.fetchall():
                    writer.writerow(row[:2] + astuple(row[2]))
        except OSError as exc:
            raise LocalCallbookExportError(str(exc)) from None

    def update_entry(self, callsign: str, data: LocalCallbookData, mode: UpdateMode = UpdateMode.Overwrite):
        """Stores or updates a set of callbook data for a callsign
        if an entry allready exists the existing data is used depending on the update mode
        :param callsign: the callsign to associate the data to
        :param data: the callbook date to store (existing data is complemented)
        :param mode: overwrite with new data, complement existing data, substitute completly with new data
        """
        callsign = callsign.upper()
        if mode != UpdateMode.Substitute:
            old_data = self.lookup(callsign)
            if old_data:
                old_data = asdict(old_data[1])
                data_dict = asdict(data)
                if mode == UpdateMode.Complement:
                    # Try to fill missing fields with new data
                    for f in old_data:
                        if not old_data[f]:
                            old_data[f] = data_dict[f]
                    data = LocalCallbookData(**old_data)
                else:
                    # Try to fill missing new fields with known data
                    for f in data_dict:
                        if not data_dict[f]:
                            data_dict[f] = old_data[f]
                    data = LocalCallbookData(**data_dict)

            self.log.info(f'Updating or adding {callsign}...')
            cur = self.__db__.execute('UPDATE callbook SET recorded=?, data=? '
                                      'WHERE callsign=?',
                                      (get_cur_dt(), data, callsign))
            if cur.rowcount < 1:
                self.__db__.execute('INSERT INTO callbook(callsign, recorded, data) VALUES(?,?,?)',
                                    (callsign, get_cur_dt(), data))
        else:
            self.log.info(f'Adding or replacing {callsign}...')
            self.__db__.execute('INSERT OR REPLACE INTO callbook(callsign, recorded, data) VALUES(?,?,?)',
                                (callsign, get_cur_dt(), data))

        self.__db__.commit()

    def import_history(self, filename: str):
        self.log.info(f'Importing history from CSV "{filename}"...')
        try:
            with open(filename, encoding='utf-8') as cf:
                reader = csv.DictReader(cf, delimiter=self.__csv_delimiter__)
                d_fields = [f.name for f in fields(CallHistoryData)]
                for hf in reader.fieldnames:
                    if hf not in ('contest', 'callsign', 'recorded') and hf not in d_fields:
                        self.log.warning(f'Unsupported field "{hf}" in history import')
                for row in reader:
                    contest = row.pop('contest')
                    callsign = row.pop('callsign')
                    row.pop('recorded', None)
                    for f in tuple(row.keys()):
                        if not f in d_fields:
                            row.pop(f)
                    if row:
                        self.update_history(contest, callsign, CallHistoryData(**row))
        except KeyError as exc:
            raise LocalCallbookImportError(f'Column {exc} is missing in history import') from None
        except OSError as exc:
            raise LocalCallbookImportError(str(exc)) from None

    def export_history(self, filename: str, contest: str = None):
        self.log.info(f'Exporting history to CSV "{filename}"...')
        try:
            with open(filename, 'w', newline='', encoding='utf-8') as cf:
                writer = csv.writer(cf, delimiter=self.__csv_delimiter__)

                # Write header
                writer.writerow(['contest', 'callsign', 'recorded'] + [f.name for f in fields(CallHistoryData)])

                if contest:
                    cur = self.__db__.execute('SELECT * FROM history '
                                              'WHERE contest=? '
                                              'ORDER BY callsign', (contest,))
                else:
                    cur = self.__db__.execute('SELECT * FROM history '
                                              'ORDER BY contest, callsign')
                row: tuple[str, str, str, CallHistoryData]
                for row in cur.fetchall():
                    writer.writerow(row[:3] + astuple(row[3]))
        except OSError as exc:
            raise LocalCallbookExportError(str(exc))

    def update_history(self, contest: str, callsign: str, data: CallHistoryData):
        """Stores or updates a set of contest call history data
        Existing data sets are updated as whole
        :param contest: the contest to associate the data to
        :param callsign: the callsign to associate the data to
        :param data: the callbook date to store
        """
        callsign = callsign.upper()
        self.log.info(f'Inserting or updating {callsign} for {contest}...')
        cur = self.__db__.execute('UPDATE history SET recorded=?, data=? '
                                  'WHERE contest=? and callsign=?',
                                  (get_cur_dt(), data, contest, callsign))
        if cur.rowcount < 1:
            self.__db__.execute('INSERT INTO history(contest, callsign, recorded, data) '
                                'VALUES(?,?,?,?)',
                                (contest, callsign, get_cur_dt(), data))
        self.__db__.commit()

    def lookup(self, callsign: str, any_fix=False) -> LocalCallbookResult | None:
        """Searches for data for a callsign
        :param callsign: the callsing to search for
        :param any_fix: search for callsign with any prefix/suffix on no result
        :return: a tuple of callsign and data
        """
        callsign = callsign.upper()
        cur = self.__db__.execute('SELECT callsign, data FROM callbook '
                                  'WHERE callsign=? '
                                  'ORDER BY recorded DESC',
                                  (callsign,))

        res = cur.fetchone()
        if not res and any_fix:
            call_check = check_call(callsign)
            if call_check:
                callsign = call_check[1]
                self.log.debug(f'Searching for any suffix for {callsign}...')
                cur = self.__db__.execute('SELECT callsign, data FROM callbook '
                                          'WHERE callsign LIKE ? or callsign LIKE ? or callsign LIKE ? '
                                          'ORDER BY recorded DESC',
                                          (f'{callsign}/%', f'%/{callsign}', f'%/{callsign}/%'))
                res = cur.fetchone()

        return LocalCallbookResult(*res) if res else None

    def lookup_history(self, contest: str, callsign: str,
                       any_fix=False, any_contest=False) -> CallHistoryResult | None:
        """Searches for data for a callsign in a contest
        If any_suffix and any_contest are used together they are tried in that order
        :param contest: the contest to search for
        :param callsign: the callsing to search for
        :param any_fix: search for callsign with any prefix/suffix on no result
        :param any_contest: search for callsign in any contest on no result (returns latests result)
        :return: a tuple of contest, callsign, history data, callbook data (if available)
        """
        callsign = callsign.upper()
        cur = self.__db__.execute('SELECT contest, callsign, history_data, callbook_data FROM history_callbook '
                                  'WHERE contest=? and callsign=? '
                                  'ORDER BY recorded DESC',
                                  (contest, callsign))

        res = cur.fetchone()
        if not res and any_fix:
            self.log.debug('Searching history for any suffix...')
            cur = self.__db__.execute('SELECT contest, callsign, history_data, callbook_data FROM history_callbook '
                                      'WHERE contest=? and (callsign LIKE ? or callsign LIKE ? or callsign LIKE ?) '
                                      'ORDER BY recorded DESC',
                                      (contest, f'{callsign}/%', f'%/{callsign}', f'%/{callsign}/%'))
            res = cur.fetchone()

        if cur.rowcount < 1 and any_contest:
            self.log.debug('Searching history in any contest...')
            cur = self.__db__.execute('SELECT contest, callsign, history_data, callbook_data FROM history_callbook '
                                      'WHERE callsign=? '
                                      'ORDER BY recorded DESC',
                                      (callsign,))
            res = cur.fetchone()

        return CallHistoryResult(*res) if res else None

    def close(self):
        """Explicitly close the database after running a cleanup and optimisation"""
        self.log.info('Reducing and optimising database...')
        self.__db__.execute('VACUUM;')
        self.__db__.execute('PRAGMA optimize;')
        self.log.info('Closing database...')
        self.__db__.close()


__all__ = [LocalCallbook, LocalCallbookData, CallHistoryData,
           LocalCallbookExportError, LocalCallbookImportError, LocalCallbookDatabaseError]



from typing import Iterable
from PyQt6 import QtWidgets, QtCore

from . import SelectWidget_ui


class SelectWidget(QtWidgets.QWidget, SelectWidget_ui.Ui_SelectWidget):
    selectionChanged = QtCore.pyqtSignal()

    def __init__(self, parent, items: Iterable[str], title_enabled: str = '', title_disabled: str = ''):
        super().__init__(parent)
        self.setupUi(self)

        self.__items__: list[str]
        self.items = items

        if title_enabled:
            self.labelEnabled.setText(title_enabled)
        if title_disabled:
            self.labelDisabled.setText(title_disabled)

    def disableItem(self):
        for item in self.enabledListWidget.selectedItems():
            self.disabledListWidget.insertItem(0, item.text())
            self.enabledListWidget.takeItem(self.enabledListWidget.row(item))

        self.disabledListWidget.sortItems()
        self.selectionChanged.emit()

    def enableItem(self):
        for item in self.disabledListWidget.selectedItems():
            self.enabledListWidget.insertItem(0, item.text())
            self.disabledListWidget.takeItem(self.disabledListWidget.row(item))

        self.enabledListWidget.sortItems()
        self.selectionChanged.emit()

    def clear(self):
        self.enabledListWidget.clear()
        self.disabledListWidget.clear()

        for i, it in enumerate(self.__items__, 1):
            self.enabledListWidget.addItem(f'{i:02d} - {it}')

        self.sortItems()
        self.selectionChanged.emit()

    @property
    def items(self) -> list[str]:
        """Return all available items"""
        return self.__items__

    @items.setter
    def items(self, items: Iterable[str]):
        """Set available items"""
        self.__items__ = list(items)
        self.clear()

    @property
    def itemsEnabled(self) -> list[str]:
        """Return enabled items"""
        return [i.text().split('-')[1].strip() for i in
                self.enabledListWidget.findItems('.*', QtCore.Qt.MatchFlag.MatchRegularExpression)]

    @property
    def indexesEnabled(self) -> list[int]:
        """Return enabled items indexes"""
        return [int(i.text().split('-')[0].strip()) for i in
                self.enabledListWidget.findItems('.*', QtCore.Qt.MatchFlag.MatchRegularExpression)]

    @property
    def indexesDisabled(self) -> list[int]:
        """Return disabled items indexes for compatibility"""
        return [int(i.text().split('-')[0].strip()) for i in
                self.disabledListWidget.findItems('.*', QtCore.Qt.MatchFlag.MatchRegularExpression)]

    @itemsEnabled.setter
    def itemsEnabled(self, items: Iterable[str]):
        self.enabledListWidget.clear()
        self.disabledListWidget.clear()

        for i, it in enumerate(self.__items__, 1):
            if it in items:
                self.enabledListWidget.addItem(f'{i:02d} - {it}')
            else:
                self.disabledListWidget.addItem(f'{i:02d} - {it}')

        self.sortItems()
        self.selectionChanged.emit()

    @indexesEnabled.setter
    def indexesEnabled(self, indexes: Iterable[int]):
        """Set disabled items by indexes"""

        self.enabledListWidget.clear()
        self.disabledListWidget.clear()

        for i, it in enumerate(self.__items__, 1):
            if i in indexes:
                self.enabledListWidget.addItem(f'{i:02d} - {it}')
            else:
                self.disabledListWidget.addItem(f'{i:02d} - {it}')

        self.sortItems()
        self.selectionChanged.emit()

    @indexesDisabled.setter
    def indexesDisabled(self, indexes: Iterable[int]):
        """Set disabled items by indexes for compatibility"""

        self.enabledListWidget.clear()
        self.disabledListWidget.clear()

        for i, it in enumerate(self.__items__, 1):
            if i in indexes:
                self.disabledListWidget.addItem(f'{i:02d} - {it}')
            else:
                self.enabledListWidget.addItem(f'{i:02d} - {it}')

        self.sortItems()
        self.selectionChanged.emit()

    def sortItems(self):
        self.enabledListWidget.sortItems()
        self.disabledListWidget.sortItems()



# DragonLog © 2023-2024 by Andreas Schawo is licensed under CC BY-SA 4.0.
# To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/

from . import DragonLog

DragonLog.main()



# Form implementation generated from reading ui file 'DragonLog_AppSelect.ui'
#
# Created by: PyQt6 UI code generator 6.9.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_appSelectDialog(object):
    def setupUi(self, appSelectDialog):
        appSelectDialog.setObjectName("appSelectDialog")
        appSelectDialog.resize(225, 200)
        self.verticalLayout = QtWidgets.QVBoxLayout(appSelectDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(parent=appSelectDialog)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.wsjtxRadioButton = QtWidgets.QRadioButton(parent=appSelectDialog)
        self.wsjtxRadioButton.setObjectName("wsjtxRadioButton")
        self.verticalLayout.addWidget(self.wsjtxRadioButton)
        self.js8callRadioButton = QtWidgets.QRadioButton(parent=appSelectDialog)
        self.js8callRadioButton.setObjectName("js8callRadioButton")
        self.verticalLayout.addWidget(self.js8callRadioButton)
        self.fldigiRadioButton = QtWidgets.QRadioButton(parent=appSelectDialog)
        self.fldigiRadioButton.setObjectName("fldigiRadioButton")
        self.verticalLayout.addWidget(self.fldigiRadioButton)
        self.otherRadioButton = QtWidgets.QRadioButton(parent=appSelectDialog)
        self.otherRadioButton.setChecked(True)
        self.otherRadioButton.setObjectName("otherRadioButton")
        self.verticalLayout.addWidget(self.otherRadioButton)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=appSelectDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(appSelectDialog)
        self.buttonBox.accepted.connect(appSelectDialog.accept) # type: ignore
        self.buttonBox.rejected.connect(appSelectDialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(appSelectDialog)

    def retranslateUi(self, appSelectDialog):
        _translate = QtCore.QCoreApplication.translate
        self.label.setText(_translate("appSelectDialog", "Select the application"))
        self.wsjtxRadioButton.setText(_translate("appSelectDialog", "WSJT-X"))
        self.js8callRadioButton.setText(_translate("appSelectDialog", "JS8Call"))
        self.fldigiRadioButton.setText(_translate("appSelectDialog", "fldigi"))
        self.otherRadioButton.setText(_translate("appSelectDialog", "Other"))



# DragonLog (c) 2023-2024 by Andreas Schawo is licensed under CC BY-SA 4.0.
# To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/

import math

import maidenhead


def distance(pos1: str, pos2: str) -> int:
    """Calculate distance between two maidenhead locators

    :param pos1: first maidenhead locator
    :param pos2: second maidenhead locator
    :return: distance in km
    """

    if not type(pos1) is str or not type(pos2) is str or len(pos1)%2 != 0 or len(pos2)%2 != 0:
        raise Exception('Maidenhead locators must be strings of 2-8 chars')

    try:
        pos1 = maidenhead.to_location(pos1, True)
        pos2 = maidenhead.to_location(pos2, True)

        mlat = math.radians(pos1[0])
        mlon = math.radians(pos1[1])
        plat = math.radians(pos2[0])
        plon = math.radians(pos2[1])

        return int(6371.01 * math.acos(
            math.sin(mlat) * math.sin(plat) + math.cos(mlat) * math.cos(plat) * math.cos(mlon - plon)))
    except ValueError as exc:
        raise Exception(exc)



# Form implementation generated from reading ui file 'ContestDlg.ui'
#
# Created by: PyQt6 UI code generator 6.9.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_ContestDialog(object):
    def setupUi(self, ContestDialog):
        ContestDialog.setObjectName("ContestDialog")
        ContestDialog.resize(554, 516)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(ContestDialog)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(parent=ContestDialog)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label)
        self.contestComboBox = QtWidgets.QComboBox(parent=ContestDialog)
        self.contestComboBox.setObjectName("contestComboBox")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.contestComboBox)
        self.label_2 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_2)
        self.bandComboBox = QtWidgets.QComboBox(parent=ContestDialog)
        self.bandComboBox.setObjectName("bandComboBox")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.bandComboBox)
        self.label_3 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_3)
        self.fromDateEdit = QtWidgets.QDateEdit(parent=ContestDialog)
        self.fromDateEdit.setMinimumDate(QtCore.QDate(1900, 1, 1))
        self.fromDateEdit.setCurrentSection(QtWidgets.QDateTimeEdit.Section.YearSection)
        self.fromDateEdit.setCalendarPopup(True)
        self.fromDateEdit.setObjectName("fromDateEdit")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.fromDateEdit)
        self.label_4 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_4)
        self.toDateEdit = QtWidgets.QDateEdit(parent=ContestDialog)
        self.toDateEdit.setCalendarPopup(True)
        self.toDateEdit.setObjectName("toDateEdit")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.ItemRole.FieldRole, self.toDateEdit)
        self.line_2 = QtWidgets.QFrame(parent=ContestDialog)
        self.line_2.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_2.setObjectName("line_2")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.ItemRole.FieldRole, self.line_2)
        self.label_6 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_6.setObjectName("label_6")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_6)
        self.callLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.callLineEdit.setObjectName("callLineEdit")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.ItemRole.FieldRole, self.callLineEdit)
        self.label_11 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_11.setObjectName("label_11")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_11)
        self.locatorLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.locatorLineEdit.setObjectName("locatorLineEdit")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.ItemRole.FieldRole, self.locatorLineEdit)
        self.label_7 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_7.setObjectName("label_7")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_7)
        self.nameLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.nameLineEdit.setObjectName("nameLineEdit")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.ItemRole.FieldRole, self.nameLineEdit)
        self.label_10 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_10.setObjectName("label_10")
        self.formLayout.setWidget(8, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_10)
        self.emailLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.emailLineEdit.setObjectName("emailLineEdit")
        self.formLayout.setWidget(8, QtWidgets.QFormLayout.ItemRole.FieldRole, self.emailLineEdit)
        self.label_5 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(9, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_5)
        self.streetLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.streetLineEdit.setObjectName("streetLineEdit")
        self.formLayout.setWidget(9, QtWidgets.QFormLayout.ItemRole.FieldRole, self.streetLineEdit)
        self.label_8 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_8.setObjectName("label_8")
        self.formLayout.setWidget(11, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_8)
        self.cityLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.cityLineEdit.setObjectName("cityLineEdit")
        self.formLayout.setWidget(11, QtWidgets.QFormLayout.ItemRole.FieldRole, self.cityLineEdit)
        self.specificLabel = QtWidgets.QLabel(parent=ContestDialog)
        self.specificLabel.setObjectName("specificLabel")
        self.formLayout.setWidget(13, QtWidgets.QFormLayout.ItemRole.LabelRole, self.specificLabel)
        self.specificLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.specificLineEdit.setObjectName("specificLineEdit")
        self.formLayout.setWidget(13, QtWidgets.QFormLayout.ItemRole.FieldRole, self.specificLineEdit)
        self.label_9 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_9.setObjectName("label_9")
        self.formLayout.setWidget(14, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_9)
        self.clubLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.clubLineEdit.setObjectName("clubLineEdit")
        self.formLayout.setWidget(14, QtWidgets.QFormLayout.ItemRole.FieldRole, self.clubLineEdit)
        self.label_15 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_15.setObjectName("label_15")
        self.formLayout.setWidget(16, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_15)
        self.label_13 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_13.setObjectName("label_13")
        self.formLayout.setWidget(18, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_13)
        self.label_12 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_12.setObjectName("label_12")
        self.formLayout.setWidget(19, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_12)
        self.opComboBox = QtWidgets.QComboBox(parent=ContestDialog)
        self.opComboBox.setObjectName("opComboBox")
        self.formLayout.setWidget(16, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opComboBox)
        self.powerComboBox = QtWidgets.QComboBox(parent=ContestDialog)
        self.powerComboBox.setObjectName("powerComboBox")
        self.formLayout.setWidget(18, QtWidgets.QFormLayout.ItemRole.FieldRole, self.powerComboBox)
        self.modeComboBox = QtWidgets.QComboBox(parent=ContestDialog)
        self.modeComboBox.setObjectName("modeComboBox")
        self.formLayout.setWidget(19, QtWidgets.QFormLayout.ItemRole.FieldRole, self.modeComboBox)
        self.line_3 = QtWidgets.QFrame(parent=ContestDialog)
        self.line_3.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_3.setObjectName("line_3")
        self.formLayout.setWidget(15, QtWidgets.QFormLayout.ItemRole.FieldRole, self.line_3)
        self.label_23 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_23.setObjectName("label_23")
        self.formLayout.setWidget(10, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_23)
        self.zipLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.zipLineEdit.setObjectName("zipLineEdit")
        self.formLayout.setWidget(10, QtWidgets.QFormLayout.ItemRole.FieldRole, self.zipLineEdit)
        self.label_24 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_24.setObjectName("label_24")
        self.formLayout.setWidget(12, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_24)
        self.countryLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.countryLineEdit.setObjectName("countryLineEdit")
        self.formLayout.setWidget(12, QtWidgets.QFormLayout.ItemRole.FieldRole, self.countryLineEdit)
        self.operatorsLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.operatorsLineEdit.setEnabled(False)
        self.operatorsLineEdit.setObjectName("operatorsLineEdit")
        self.formLayout.setWidget(17, QtWidgets.QFormLayout.ItemRole.FieldRole, self.operatorsLineEdit)
        self.label_25 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_25.setObjectName("label_25")
        self.formLayout.setWidget(17, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_25)
        self.verticalLayout_2.addLayout(self.formLayout)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.horizontalLayout_3.addLayout(self.verticalLayout_2)
        self.line_1 = QtWidgets.QFrame(parent=ContestDialog)
        self.line_1.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_1.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_1.setObjectName("line_1")
        self.horizontalLayout_3.addWidget(self.line_1)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.ediGroupBox = QtWidgets.QGroupBox(parent=ContestDialog)
        self.ediGroupBox.setEnabled(False)
        self.ediGroupBox.setObjectName("ediGroupBox")
        self.formLayout_3 = QtWidgets.QFormLayout(self.ediGroupBox)
        self.formLayout_3.setObjectName("formLayout_3")
        self.antennaLineEdit = QtWidgets.QLineEdit(parent=self.ediGroupBox)
        self.antennaLineEdit.setObjectName("antennaLineEdit")
        self.formLayout_3.setWidget(5, QtWidgets.QFormLayout.ItemRole.FieldRole, self.antennaLineEdit)
        self.rigLineEdit = QtWidgets.QLineEdit(parent=self.ediGroupBox)
        self.rigLineEdit.setObjectName("rigLineEdit")
        self.formLayout_3.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.rigLineEdit)
        self.label_19 = QtWidgets.QLabel(parent=self.ediGroupBox)
        self.label_19.setObjectName("label_19")
        self.formLayout_3.setWidget(5, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_19)
        self.powerSpinBox = QtWidgets.QSpinBox(parent=self.ediGroupBox)
        self.powerSpinBox.setMaximum(999)
        self.powerSpinBox.setObjectName("powerSpinBox")
        self.formLayout_3.setWidget(3, QtWidgets.QFormLayout.ItemRole.FieldRole, self.powerSpinBox)
        self.label_18 = QtWidgets.QLabel(parent=self.ediGroupBox)
        self.label_18.setObjectName("label_18")
        self.formLayout_3.setWidget(3, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_18)
        self.label_20 = QtWidgets.QLabel(parent=self.ediGroupBox)
        self.label_20.setObjectName("label_20")
        self.formLayout_3.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_20)
        self.qthLineEdit = QtWidgets.QLineEdit(parent=self.ediGroupBox)
        self.qthLineEdit.setObjectName("qthLineEdit")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.qthLineEdit)
        self.label_21 = QtWidgets.QLabel(parent=self.ediGroupBox)
        self.label_21.setObjectName("label_21")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_21)
        self.label_16 = QtWidgets.QLabel(parent=self.ediGroupBox)
        self.label_16.setObjectName("label_16")
        self.formLayout_3.setWidget(6, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_16)
        self.antAboveGroundSpinBox = QtWidgets.QSpinBox(parent=self.ediGroupBox)
        self.antAboveGroundSpinBox.setMinimum(-1)
        self.antAboveGroundSpinBox.setMaximum(499)
        self.antAboveGroundSpinBox.setStepType(QtWidgets.QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.antAboveGroundSpinBox.setProperty("value", -1)
        self.antAboveGroundSpinBox.setObjectName("antAboveGroundSpinBox")
        self.formLayout_3.setWidget(6, QtWidgets.QFormLayout.ItemRole.FieldRole, self.antAboveGroundSpinBox)
        self.label_17 = QtWidgets.QLabel(parent=self.ediGroupBox)
        self.label_17.setObjectName("label_17")
        self.formLayout_3.setWidget(7, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_17)
        self.antAboveSeaSpinBox = QtWidgets.QSpinBox(parent=self.ediGroupBox)
        self.antAboveSeaSpinBox.setMinimum(-1)
        self.antAboveSeaSpinBox.setMaximum(9999)
        self.antAboveSeaSpinBox.setStepType(QtWidgets.QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.antAboveSeaSpinBox.setProperty("value", -1)
        self.antAboveSeaSpinBox.setObjectName("antAboveSeaSpinBox")
        self.formLayout_3.setWidget(7, QtWidgets.QFormLayout.ItemRole.FieldRole, self.antAboveSeaSpinBox)
        self.line = QtWidgets.QFrame(parent=self.ediGroupBox)
        self.line.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line.setObjectName("line")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.line)
        self.line_4 = QtWidgets.QFrame(parent=self.ediGroupBox)
        self.line_4.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_4.setObjectName("line_4")
        self.formLayout_3.setWidget(4, QtWidgets.QFormLayout.ItemRole.FieldRole, self.line_4)
        self.verticalLayout.addWidget(self.ediGroupBox)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem1)
        self.label_14 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_14.setObjectName("label_14")
        self.verticalLayout.addWidget(self.label_14)
        self.soapPlainTextEdit = QtWidgets.QPlainTextEdit(parent=ContestDialog)
        self.soapPlainTextEdit.setMinimumSize(QtCore.QSize(0, 40))
        self.soapPlainTextEdit.setMaximumSize(QtCore.QSize(16777215, 60))
        self.soapPlainTextEdit.setObjectName("soapPlainTextEdit")
        self.verticalLayout.addWidget(self.soapPlainTextEdit)
        spacerItem2 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Minimum)
        self.verticalLayout.addItem(spacerItem2)
        self.label_22 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_22.setObjectName("label_22")
        self.verticalLayout.addWidget(self.label_22)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.exportPathLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.exportPathLineEdit.setObjectName("exportPathLineEdit")
        self.horizontalLayout_2.addWidget(self.exportPathLineEdit)
        self.choosePathToolButton = QtWidgets.QToolButton(parent=ContestDialog)
        self.choosePathToolButton.setObjectName("choosePathToolButton")
        self.horizontalLayout_2.addWidget(self.choosePathToolButton)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        spacerItem3 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Minimum)
        self.verticalLayout.addItem(spacerItem3)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem4)
        self.exportPushButton = QtWidgets.QPushButton(parent=ContestDialog)
        self.exportPushButton.setEnabled(False)
        self.exportPushButton.setObjectName("exportPushButton")
        self.horizontalLayout.addWidget(self.exportPushButton)
        self.cancelPushButton = QtWidgets.QPushButton(parent=ContestDialog)
        self.cancelPushButton.setObjectName("cancelPushButton")
        self.horizontalLayout.addWidget(self.cancelPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_3.addLayout(self.verticalLayout)

        self.retranslateUi(ContestDialog)
        self.exportPushButton.clicked.connect(ContestDialog.accept) # type: ignore
        self.cancelPushButton.clicked.connect(ContestDialog.reject) # type: ignore
        self.contestComboBox.currentTextChanged['QString'].connect(ContestDialog.contestChanged) # type: ignore
        self.fromDateEdit.dateChanged['QDate'].connect(ContestDialog.fromDateChanged) # type: ignore
        self.choosePathToolButton.clicked.connect(ContestDialog.choosePath) # type: ignore
        self.callLineEdit.textChanged['QString'].connect(ContestDialog.callChanged) # type: ignore
        self.locatorLineEdit.textChanged['QString'].connect(ContestDialog.locatorChanged) # type: ignore
        self.nameLineEdit.textChanged['QString'].connect(ContestDialog.nameChanged) # type: ignore
        self.cityLineEdit.textChanged['QString'].connect(ContestDialog.cityChanged) # type: ignore
        self.streetLineEdit.textChanged['QString'].connect(ContestDialog.streetChanged) # type: ignore
        self.emailLineEdit.textChanged['QString'].connect(ContestDialog.emailChanged) # type: ignore
        self.clubLineEdit.textChanged['QString'].connect(ContestDialog.clubChanged) # type: ignore
        self.specificLineEdit.textChanged['QString'].connect(ContestDialog.specificChanged) # type: ignore
        self.zipLineEdit.textChanged['QString'].connect(ContestDialog.zipChanged) # type: ignore
        self.opComboBox.currentTextChanged['QString'].connect(ContestDialog.operatorChanged) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(ContestDialog)

    def retranslateUi(self, ContestDialog):
        _translate = QtCore.QCoreApplication.translate
        ContestDialog.setWindowTitle(_translate("ContestDialog", "Contest Export"))
        self.label.setText(_translate("ContestDialog", "Contest"))
        self.label_2.setText(_translate("ContestDialog", "Band"))
        self.label_3.setText(_translate("ContestDialog", "Date from"))
        self.fromDateEdit.setDisplayFormat(_translate("ContestDialog", "yyyy-MM-dd"))
        self.label_4.setText(_translate("ContestDialog", "Date to"))
        self.toDateEdit.setDisplayFormat(_translate("ContestDialog", "yyyy-MM-dd"))
        self.label_6.setText(_translate("ContestDialog", "Callsign"))
        self.callLineEdit.setPlaceholderText(_translate("ContestDialog", "DX1XX"))
        self.label_11.setText(_translate("ContestDialog", "Locator"))
        self.locatorLineEdit.setPlaceholderText(_translate("ContestDialog", "AA11aa"))
        self.label_7.setText(_translate("ContestDialog", "Fullname"))
        self.nameLineEdit.setPlaceholderText(_translate("ContestDialog", "Max Example"))
        self.label_10.setText(_translate("ContestDialog", "eMail"))
        self.emailLineEdit.setPlaceholderText(_translate("ContestDialog", "dx1xx@darc.de"))
        self.label_5.setText(_translate("ContestDialog", "Street"))
        self.streetLineEdit.setPlaceholderText(_translate("ContestDialog", "Street 1"))
        self.label_8.setText(_translate("ContestDialog", "City"))
        self.cityLineEdit.setPlaceholderText(_translate("ContestDialog", "Berlin"))
        self.specificLabel.setText(_translate("ContestDialog", "Specific"))
        self.specificLineEdit.setPlaceholderText(_translate("ContestDialog", "Contest specific information"))
        self.label_9.setText(_translate("ContestDialog", "Club"))
        self.clubLineEdit.setPlaceholderText(_translate("ContestDialog", "DARC"))
        self.label_15.setText(_translate("ContestDialog", "Operator"))
        self.label_13.setText(_translate("ContestDialog", "Power"))
        self.label_12.setText(_translate("ContestDialog", "Mode"))
        self.label_23.setText(_translate("ContestDialog", "Zip"))
        self.zipLineEdit.setPlaceholderText(_translate("ContestDialog", "11111"))
        self.label_24.setText(_translate("ContestDialog", "Country"))
        self.countryLineEdit.setPlaceholderText(_translate("ContestDialog", "Deutschland"))
        self.operatorsLineEdit.setPlaceholderText(_translate("ContestDialog", "DX1XX DY1YY DX2XX DY2YY"))
        self.label_25.setText(_translate("ContestDialog", "Operators"))
        self.ediGroupBox.setTitle(_translate("ContestDialog", "Additional EDI data"))
        self.label_19.setText(_translate("ContestDialog", "Antenna"))
        self.powerSpinBox.setSpecialValueText(_translate("ContestDialog", "n.a."))
        self.powerSpinBox.setSuffix(_translate("ContestDialog", " W"))
        self.label_18.setText(_translate("ContestDialog", "Power"))
        self.label_20.setText(_translate("ContestDialog", "Rig"))
        self.label_21.setText(_translate("ContestDialog", "Contest QTH"))
        self.label_16.setText(_translate("ContestDialog", "Ant. above ground"))
        self.antAboveGroundSpinBox.setSpecialValueText(_translate("ContestDialog", "n.a."))
        self.antAboveGroundSpinBox.setSuffix(_translate("ContestDialog", " m"))
        self.label_17.setText(_translate("ContestDialog", "Ant. above sea lvl."))
        self.antAboveSeaSpinBox.setSpecialValueText(_translate("ContestDialog", "n.a."))
        self.antAboveSeaSpinBox.setSuffix(_translate("ContestDialog", " m"))
        self.label_14.setText(_translate("ContestDialog", "Soapbox"))
        self.soapPlainTextEdit.setPlaceholderText(_translate("ContestDialog", "Extra information"))
        self.label_22.setText(_translate("ContestDialog", "Export path"))
        self.exportPathLineEdit.setPlaceholderText(_translate("ContestDialog", "Export path"))
        self.choosePathToolButton.setText(_translate("ContestDialog", "..."))
        self.exportPushButton.setText(_translate("ContestDialog", "Export"))
        self.cancelPushButton.setText(_translate("ContestDialog", "Cancel"))



# DragonLog (c) 2025 by Andreas Schawo is licensed under CC BY-SA 4.0.
# To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/

import os
import socket
import logging
import platform
import subprocess
import sys

from PyQt6 import QtCore

from .Logger import Logger


class NoExecutableFoundException(Exception):
    pass


class RigctldNotConfiguredException(Exception):
    pass


class CATSettingsMissingException(Exception):
    pass


class RigctldExecutionException(Exception):
    pass


RIGCTL_ECODE = {
    0: 'Command completed successfully',
    -1: 'Invalid parameter',
    -2: 'Invalid configuration',
    -3: 'Memory shortage',
    -4: 'Feature not implemented',
    -5: 'Communication timed out',
    -6: 'IO error',
    -7: 'Internal Hamlib error',
    -8: 'Protocol error',
    -9: 'Command rejected by the rig',
    -10: 'Command performed, but arg truncated, result not guaranteed',
    -11: 'Feature not available',
    -12: 'Target VFO unaccessible',
    -13: 'Communication bus error',
    -14: 'Communication bus collision',
    -15: 'NULL RIG handle or invalid pointer parameter',
    -16: 'Invalid VFO',
    -17: 'Argument out of domain of func',
    -18: 'Function deprecated',
    -19: 'Security error password not provided or crypto failure',
    -20: 'Rig is not powered on',
    -21: 'Limit exceeded',
    -22: 'Access denied',
}


class RigControl(QtCore.QObject):
    frequencyChanged = QtCore.pyqtSignal(float)
    bandChanged = QtCore.pyqtSignal(str)
    modeChanged = QtCore.pyqtSignal(str)
    submodeChanged = QtCore.pyqtSignal(str)
    powerChanged = QtCore.pyqtSignal(int)
    statusChanged = QtCore.pyqtSignal(bool)

    def __init__(self, parent, settings: QtCore.QSettings, logger: Logger,
                 bands: dict, modes: dict):
        super().__init__(parent)

        # From QSOForm
        self.log = logging.getLogger('RigControl')
        self.log.addHandler(logger)
        self.log.setLevel(logger.loglevel)
        self.logger = logger
        self.log.debug('Initialising...')

        self.settings = settings

        self.bands = bands
        self.modes = modes

        self.rig_modes = {'USB': ('SSB', 'USB'),
                          'LSB': ('SSB', 'LSB'),
                          'CW': ('CW', ''),
                          'CWR': ('CW', ''),
                          'RTTY': ('RTTY', ''),
                          'RTTYR': ('RTTY', ''),
                          'AM': ('AM', ''),
                          'FM': ('FM', ''),
                          'FMN': ('FM', ''),
                          'WFM': ('FM', ''),
                          'PKTUSB': ('SSB', 'USB'),
                          'PKTLSB': ('SSB', 'LSB'),
                          }
        self.__last_mode__ = ''
        self.__last_band__ = ''
        self.__last_freq__ = 0.0
        self.__last_pwr_lvl__ = ''
        self.__last_pwr__ = 0

        self.__rig_ids__: dict = {}
        self.__rigs__: dict[str, list[str]] = {}
        self.__rigctld_path__: os.PathLike | None = None
        self.__rigctld__: subprocess.Popen | None = None
        self.__rig_caps__: list = []

        self.__rigctl_startupinfo__ = None
        if platform.system() == 'Windows':
            self.__rigctl_startupinfo__ = subprocess.STARTUPINFO()
            self.__rigctl_startupinfo__.dwFlags |= subprocess.STARTF_USESHOWWINDOW
            if self.settings.value('cat/rigctldPath', None):
                if self.__is_exe__(self.settings.value('cat/rigctldPath', '')):
                    self.init_hamlib(self.settings.value('cat/rigctldPath'))
                else:
                    raise NoExecutableFoundException(self.settings.value('cat/rigctldPath', ''))
        else:
            self.init_hamlib('rigctld')

        self.__refreshTimer__ = QtCore.QTimer(self)
        self.__refreshTimer__.timeout.connect(self.__refreshRigData__)
        self.__checkHamlibTimer__ = QtCore.QTimer(self)
        self.__checkHamlibTimer__.timeout.connect(self.__checkRigctld__)

    @staticmethod
    def __is_exe__(path: str) -> bool:
        return os.path.isfile(path) and os.access(path, os.X_OK)

    def __checkRigctld__(self):
        if not self.isActive():
            self.log.error('rigctld died unexpectedly')
            self.__rig_caps__ = []
            self.__checkHamlibTimer__.stop()
            self.statusChanged.emit(False)

    def isActive(self) -> bool:
        return self.__rigctld__ and not self.__rigctld__.poll()

    def init_hamlib(self, rigctld_path: str):
        if rigctld_path:
            try:
                res = subprocess.run([rigctld_path, '-l'], capture_output=True)
                stdout = str(res.stdout, sys.getdefaultencoding()).replace('\r', '')
                if res.returncode != 0 or not stdout:
                    self.log.error(f'Error executing rigctld: {self.__get_errcode__(res.returncode)}')
                    raise RigctldExecutionException(rigctld_path)
                self.log.debug('Executed rigctld to list rigs')
            except FileNotFoundError:
                self.log.warning('rigctld is not available')
                raise NoExecutableFoundException(rigctld_path)

            first = True
            rig_pos = 0
            mfr_pos = 0
            model_pos = 0
            end_pos = 0
            self.__rigs__ = {}
            self.__rig_ids__ = {}
            for rig in stdout.split('\n'):
                if first:
                    first = False
                    rig_pos = rig.index('Rig #')
                    mfr_pos = rig.index('Mfg')
                    model_pos = rig.index('Model')
                    end_pos = rig.index('Version')
                    continue
                elif not rig.strip():  # Empty line
                    continue

                rig_id = rig[rig_pos:mfr_pos - 1].strip()
                mfr_name = rig[mfr_pos:model_pos - 1].strip()
                model_name = rig[model_pos:end_pos - 1].strip()

                self.__rig_ids__[f'{mfr_name}/{model_name}'] = rig_id
                if mfr_name in self.__rigs__:
                    self.__rigs__[mfr_name].append(model_name)
                else:
                    self.__rigs__[mfr_name] = [model_name]

            self.__rigctld_path__ = rigctld_path

    # From Settings
    def __collectRigCaps__(self, rig_id: str):
        res = subprocess.run([self.__rigctld_path__, f'--model={rig_id}', '-u'],
                             capture_output=True,
                             startupinfo=self.__rigctl_startupinfo__)
        stdout = str(res.stdout, sys.getdefaultencoding()).replace('\r', '')
        self.__rig_caps__ = []
        for ln in stdout.split('\n'):
            if ln.startswith('Can '):
                cap, able = ln.split(':')
                if able.strip() == 'Y':
                    self.__rig_caps__.append(cap[4:].lower())

    @property
    def availableManufacturers(self) -> list:
        return list(self.__rigs__.keys())

    def availableRigs(self, mfr: str) -> list:
        return self.__rigs__.get(mfr, [])

    @property
    def capabilities(self) -> list:
        return self.__rig_caps__

    # noinspection PyUnresolvedReferences
    def ctrlRigctld(self, start: bool):
        if start:
            if not self.__rigctld_path__:
                self.log.warning('rigctld is not available')
                raise RigctldNotConfiguredException()

            if not self.__rigctld__:
                rig_mfr = self.settings.value('cat/rigMfr', '')
                rig_model = self.settings.value('cat/rigModel', '')
                rig_if = self.settings.value('cat/interface', '')
                rig_speed = self.settings.value('cat/baud', '')
                if not rig_mfr or not rig_model or not rig_if or not rig_speed:
                    raise CATSettingsMissingException()

                rig_id = self.__rig_ids__[f'{rig_mfr}/{rig_model}']

                self.__collectRigCaps__(rig_id)

                self.__rigctld__ = subprocess.Popen([self.__rigctld_path__,
                                                     f'--model={rig_id}',
                                                     f'--rig-file={rig_if}',
                                                     f'--serial-speed={rig_speed}',
                                                     '--listen-addr=127.0.0.1'],
                                                    startupinfo=self.__rigctl_startupinfo__)

                if self.__rigctld__.poll():
                    self.statusChanged.emit(False)
                else:
                    self.log.info(
                        f'rigctld is running with pid #{self.__rigctld__.pid} and arguments {self.__rigctld__.args}')
                    self.__checkHamlibTimer__.start(1000)
                    self.statusChanged.emit(True)
                    # Give rigctld some time on slower machines
                    self.__refreshTimer__.singleShot(500, self.startRefresh)
        else:
            self.__checkHamlibTimer__.stop()
            if self.isActive():
                os.kill(self.__rigctld__.pid, 9)
                self.log.info('Killed rigctld')
            self.__rigctld__ = None
            self.__rig_caps__ = []
            self.statusChanged.emit(False)

    # From QSOForm
    def rigctldChanged(self, state: bool):
        self.__last_mode__ = ''
        self.__last_band__ = ''
        self.__last_freq__ = 0.0
        self.__last_pwr_lvl__ = ''
        self.__last_pwr__ = 0

    @property
    def mode(self) -> str:
        return self.__last_mode__

    @property
    def band(self) -> str:
        return self.__last_band__

    @property
    def frequency(self) -> float:
        return self.__last_freq__

    @property
    def power(self) -> int:
        return self.__last_pwr__

    def startRefresh(self):
        self.__refreshTimer__.start(100)

    def setRigFreq(self, freq: int | float):
        self.sendToRig(f'set_freq {int(freq * 1000)}')

    def sendToRig(self, cmd: str):
        if not self.isActive():
            return

        try:
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                s.connect(('127.0.0.1', 4532))
                s.settimeout(1)
                try:
                    s.sendall(f'\\{cmd}\n'.encode())
                    res = s.recv(1024).decode('utf-8').strip()
                    if not res.startswith('RPRT 0'):
                        self.log.error(f'rigctld error "{cmd}": {self.__get_errcode__(res.split()[1])}')
                    else:
                        self.log.debug(f'rigctld "{cmd}" successful')
                except socket.timeout:
                    self.log.error('rigctld error: timeout in sendToRig')
        except ConnectionRefusedError:
            self.log.error('Could not connect to rigctld in sendToRig')

    @staticmethod
    def __get_errcode__(code: str | int) -> str:
        err = str(code)
        try:
            err = RIGCTL_ECODE.get(int(code), err)
        except ValueError:
            pass
        return str(err)

    # noinspection PyBroadException
    def __refreshRigData__(self):
        if self.isActive():
            try:
                with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                    s.connect(('127.0.0.1', 4532))
                    s.settimeout(1)
                    try:
                        # Get frequency
                        s.sendall(b'\\get_freq\n')
                        freq_s = s.recv(1024).decode('utf-8').strip()
                        if freq_s.startswith('RPRT'):
                            self.log.error(f'rigctld error get_freq: {self.__get_errcode__(freq_s.split()[1])}')
                            return

                        try:
                            freq = float(freq_s) / 1000
                            if freq != self.__last_freq__:
                                for b in self.bands:
                                    if freq < self.bands[b][1]:
                                        if freq > self.bands[b][0]:
                                            if b != self.__last_band__:
                                                self.bandChanged.emit(b)
                                                self.log.info(f'CAT changed band to {b}')
                                                self.__last_band__ = b
                                                self.__last_mode__ = ''
                                        break
                                self.frequencyChanged.emit(freq)
                                self.__last_freq__ = freq
                        except Exception:
                            pass

                        # Get mode
                        s.sendall(b'\\get_mode\n')
                        mode_s = s.recv(1024).decode('utf-8').strip()
                        if mode_s.startswith('RPRT'):
                            self.log.error(f'rigctld error get_mode: {self.__get_errcode__(mode_s.split()[1])}')
                            return

                        try:
                            mode, passband = [v.strip() for v in mode_s.split('\n')]
                            if mode in self.rig_modes and mode != self.__last_mode__:
                                self.modeChanged.emit(self.rig_modes[mode][0])
                                self.log.info(f'CAT changed mode to {self.rig_modes[mode][0]}')
                                if self.rig_modes[mode][1]:
                                    self.submodeChanged.emit(self.rig_modes[mode][1])
                                self.__last_mode__ = mode
                        except Exception:
                            pass

                        # Get power
                        if 'get level' in self.__rig_caps__ and 'get power2mw' in self.__rig_caps__:
                            # Get power level
                            s.sendall(b'\\get_level RFPOWER\n')
                            pwrlvl_s = s.recv(1024).decode('utf-8').strip()
                            if pwrlvl_s.startswith('RPRT'):
                                self.log.error(f'rigctld error get_level: {self.__get_errcode__(pwrlvl_s.split()[1])}')
                                return

                            if pwrlvl_s != self.__last_pwr_lvl__:
                                self.__last_pwr_lvl__ = pwrlvl_s
                                # Convert level to W
                                s.sendall(f'\\power2mW {pwrlvl_s} {freq_s} {mode}\n'.encode())
                                pwr_s = s.recv(1024).decode('utf-8').strip()
                                if pwr_s.startswith('RPRT'):
                                    self.log.error(f'rigctld error power2mW: {self.__get_errcode__(pwr_s.split()[1])}')
                                    return

                                try:
                                    self.__last_pwr__ = int(int(pwr_s) / 1000 + .9)
                                    self.powerChanged.emit(self.__last_pwr__)
                                except Exception:
                                    pass
                    except socket.timeout:
                        self.log.error('rigctld error: timeout in refreshRigData')
                        self.ctrlRigctld(False)
            except ConnectionRefusedError:
                self.log.error('Could not connect to rigctld in refreshRigData')
                self.ctrlRigctld(False)
